/********************************************************* begin of preamble
**
** Copyright (C) 2003-2012 Software- und Organisations-Service GmbH. 
** All rights reserved.
**
** This file may be used under the terms of either the 
**
**   GNU General Public License version 2.0 (GPL)
**
**   as published by the Free Software Foundation
**   http://www.gnu.org/licenses/gpl-2.0.txt and appearing in the file
**   LICENSE.GPL included in the packaging of this file. 
**
** or the
**  
**   Agreement for Purchase and Licensing
**
**   as offered by Software- und Organisations-Service GmbH
**   in the respective terms of supply that ship with this file.
**
** THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS
** IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO,
** THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR
** PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS
** BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
** CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
** SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
** INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
** CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
** ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
** POSSIBILITY OF SUCH DAMAGE.
********************************************************** end of preamble*/
package com.sos.DataExchange;

import java.io.IOException;
import java.util.HashMap;
import java.util.Properties;
import java.util.Vector;

import org.apache.log4j.Level;
import org.apache.log4j.Logger;

import sos.net.SOSMail;
import sos.net.mail.options.SOSSmtpMailOptions;

import com.sos.JSHelper.Basics.JSJobUtilities;
import com.sos.JSHelper.Exceptions.JobSchedulerException;
import com.sos.JSHelper.Options.SOSOptionJadeOperation.enuJadeOperations;
import com.sos.JSHelper.io.Files.JSFile;
import com.sos.VirtualFileSystem.DataElements.SOSFileList;
import com.sos.VirtualFileSystem.DataElements.SOSFileListEntry;
import com.sos.VirtualFileSystem.FTP.SOSFTPOptions;
import com.sos.VirtualFileSystem.Factory.VFSFactory;
import com.sos.VirtualFileSystem.Interfaces.ISOSVFSHandler;
import com.sos.VirtualFileSystem.Interfaces.ISOSVfsFileTransfer;
import com.sos.VirtualFileSystem.Interfaces.ISOSVirtualFile;
import com.sos.i18n.annotation.I18NResourceBundle;

@I18NResourceBundle(baseName = "SOSDataExchange", defaultLocale = "en")
public class SOSDataExchangeEngine extends SOSJadeMessageCodes /* I18NBase */implements Runnable, JSJobUtilities {
	private static final String	conKeyWordLAST_ERROR			= "last_error";
	private static final String	conKeywordSTATE					= "state";
	private static final String	conKeywordSUCCESSFUL_TRANSFERS	= "successful_transfers";
	private static final String	conKeywordFAILED_TRANSFERS		= "failed_transfers";
	private static final String	conKeywordSTATUS				= "status";
	private final String		conClassName					= "SOSDataExchangeEngine";
	public final String			conSVNVersion					= "$Id: SOSDataExchangeEngine.java 17819 2012-08-08 19:42:19Z kb $";

	private final Logger				logger							= Logger.getLogger(SOSDataExchangeEngine.class);
	final String				strLoggerName					= "JadeReportLog";
	private final Logger				objJadeReportLogger				= Logger.getLogger(strLoggerName);

	private ISOSVFSHandler		objVfs4Target					= null;
	private ISOSVFSHandler		objVfs4Source					= null;
	public ISOSVfsFileTransfer	objDataTargetClient				= null;
	public ISOSVfsFileTransfer	objDataSourceClient				= null;
	private SOSFTPOptions		objOptions						= null;
	private SOSFileList			objSourceFileList				= null;
	private final boolean				jump							= false;																// TODO
																																		// temporr
	private final boolean				makeDirs						= true;
	/** This parameter specifies whether zero byte files should be transferred and processed by subsequent jobs. The following settings are available */
	private final boolean				zeroByteFiles					= true;
	private final boolean				zeroByteFilesStrict				= false;
	private final boolean				zeroByteFilesRelaxed			= false;
	/** notification by e-mail in case of transfer of empty files. */
	private final String				fileNotificationTo				= "";
	private final String				fileNotificationCC				= "";
	private final String				fileNotificationBCC				= "";
	private final String				fileNotificationSubject			= "";
	private final String				fileNotificationBody			= "";
	private final String				fileZeroByteNotificationTo		= "";
	private final String				fileZeroByteNotificationCC		= "";
	private final String				fileZeroByteNotificationBCC		= "";
	private final String				fileZeroByteNotificationSubject	= "";
	private final String				fileZeroByteNotificationBody	= "";
	private JSJobUtilities		objJSJobUtilities				= this;
	// private boolean sameConnection = false;
	// private int count = 0;
	private final int					zeroByteCount					= 0;																	// temporr
	// private boolean isFilePath = false;
	// private String state = "";
	/**
	 * Resource filenames for operation = install
	 */
	@SuppressWarnings("unused")
	private final boolean				testmode						= false;

	public SOSDataExchangeEngine() throws Exception {
		init();
	}

	private void init() {

		@SuppressWarnings("unused")
		final String conMethodName = conClassName + "::init";

		logger.info(conClassName + " --- " + conSVNVersion);

	} // private void init

	/**
	 * @param settingsFile
	 * @param settingsSection
	 * @param logger
	 * @param arguments_
	 */
	public SOSDataExchangeEngine(final Properties pobjProperties) throws Exception {
		this.Options();
		// TODO Properties in die OptionsClasse weiterreichen
		// objOptions.setAllOptions(pobjProperties);
	}

	public SOSDataExchangeEngine(final SOSFTPOptions pobjOptions) throws Exception {
		objOptions = pobjOptions;
	}

	public SOSDataExchangeEngine(final HashMap<String, String> pobjJSSettings) throws Exception {
		this.Options();
		objOptions.setAllOptions(pobjJSSettings);
	}

	@SuppressWarnings("unused")
	private ISOSVFSHandler getVFS() throws Exception {
		if (objVfs4Target == null) {
			objVfs4Target = VFSFactory.getHandler(objOptions.getDataTargetType());
		}
		return objVfs4Target;
	}

	public SOSFTPOptions Options() {
		if (objOptions == null) {
			objOptions = new SOSFTPOptions();
		}
		return objOptions;
	}

	@Override
	public void run() {
		@SuppressWarnings("unused")
		final String conMethodName = conClassName + "::Run";
		try {
			this.Execute();
		}
		catch (Exception e) {
			throw new JobSchedulerException(EXCEPTION_RAISED.get(e), e);
		}
	}

	public boolean Execute() throws Exception {
		@SuppressWarnings("unused")
		final String conMethodName = conClassName + "::execute";

		VFSFactory.setParentLogger(strLoggerName);
		if (objOptions.verbose.value() == 0) {
			Logger.getRootLogger().setLevel(Level.INFO);
		}
		else {
			Logger.getRootLogger().setLevel(Level.DEBUG);
		}
		objOptions.getTextProperties().put("version", conSVNVersion);

		objOptions.log_filename.setLogger(objJadeReportLogger);
		String strT = SOSJADE_T_0010.get(); // LogFile Header
		strT = objOptions.replaceVars(strT);
		objJadeReportLogger.info(strT);

		boolean flgOK = false;
		try {
			flgOK = this.transfer();
		}
		catch (Exception e) {
			
			throw e;
		}
		finally {
			strT = SOSJADE_T_0011.get(); // LogFile Footer
			strT = objOptions.replaceVars(strT);
			objJadeReportLogger.info(strT);
			sendMail();
		}

		return flgOK;
	}

	private void sendMail() {

		@SuppressWarnings("unused")
		final String conMethodName = conClassName + "::sendMail";

		SOSSmtpMailOptions objO = objOptions.getMailOptions();
		if (objO.FileNotificationTo.isDirty() == true) {

			try {
				String strA = "";
				String strF = objOptions.log_filename.getHtmlLogFileName();
				if (strF.length() > 0) {
					strA += strF;
				}

				strF = objOptions.log_filename.Value();
				if (strF.length() > 0) {
					if (strA.length() > 0) {
						strA += ";";
					}
					strA += strF;
				}
				if (strA.length() > 0) {
					objO.attachment.Value(strA);
				}
				
				if (objO.subject.isDirty() == false) {
					String strT = "JADE: ";
					objO.subject.Value(strT);
				}

				String strM = objO.subject.Value();
				objO.subject.Value(objOptions.replaceVars(strM));

				strM = objO.body.Value();
				objO.body.Value(objOptions.replaceVars(strM));
				
				if (objO.from.isDirty() == false) {
					objO.from.Value("JADE@sos-berlin.com");
				}

				SOSMail objMail = new SOSMail(objO.host.Value());
				objMail.sendMail(objO);

			}
			catch (Exception e) {
				// TODO Auto-generated catch block
				e.printStackTrace();
			}

		}

	} // private void sendMail

	public ISOSVfsFileTransfer DataSourceClient() {

		@SuppressWarnings("unused")
		final String conMethodName = conClassName + "::DataSourceClient";

		if (objDataSourceClient == null) {
			try {
				objVfs4Source = VFSFactory.getHandler(objOptions.getDataSourceType());
				objVfs4Source.Connect(objOptions.getConnectionOptions().Source());
				objVfs4Source.Authenticate(objOptions.getConnectionOptions().Source());
				objVfs4Source.setSource();
				objVfs4Source.Options(objOptions);
			}
			catch (Exception e) {
				throw new JobSchedulerException(SOSJADE_E_0200.params(e));
			}
		}

		return objDataSourceClient;
	} // private ISOSVfsFileTransfer DataSourceClient

	/**
	 * 
	 * \brief DataTargetClient
	 * Returns an instance of the TargetClient
	 * 
	 * \details
	 *
	 * \return ISOSVfsFileTransfer
	 *
	 * @return
	 * @throws Exception
	 */
	public ISOSVfsFileTransfer DataTargetClient() throws Exception {
		@SuppressWarnings("unused")
		final String conMethodName = conClassName + "::DataTarget";
		if (objDataTargetClient == null) {
			objOptions.CheckMandatory();
			objVfs4Target = VFSFactory.getHandler(objOptions.getDataTargetType());
			objVfs4Target.Connect(objOptions.getConnectionOptions().Target());
			objVfs4Target.Authenticate(objOptions.getConnectionOptions().Target());
			objDataTargetClient = (ISOSVfsFileTransfer) objVfs4Target;
			objVfs4Target.setTarget();
			objVfs4Target.Options(objOptions);
		}
		return objDataTargetClient;
	} // private ISOSVfsFileTransfer DataTarget

	/**
	 * Send files by  .... from source to a target  
	 * 
	 * @return boolean
	 * @throws Exception
	 */
	public boolean transfer() throws Exception {
		boolean flgReturnCode = false;
		try { // to connect, authenticate and execute commands
			Options().CheckMandatory();
			setTextProperties();

			// logger.debug(Options().toString());
			objSourceFileList = new SOSFileList(objVfs4Target);
			objSourceFileList.Options(objOptions);
			objSourceFileList.StartTransaction();
			DataSourceClient();
			if (objOptions.NeedTargetClient() == true) {
				DataTargetClient();
				objSourceFileList.objDataTargetClient = objDataTargetClient;
				if (objOptions.passive_mode.value()) {
					objDataTargetClient.passive();
				}
				objDataTargetClient.TransferMode(objOptions.transfer_mode);
				objDataTargetClient.ControlEncoding(objOptions.ControlEncoding.Value());
				if (objOptions.PreFtpCommands.IsNotEmpty()) {
					// TODO Command separator as option
					String[] strA = objOptions.PreFtpCommands.Value().split(";");
					for (String strCmd : strA) {
						strCmd = objOptions.replaceVars(strCmd);
						objDataTargetClient.getHandler().ExecuteCommand(strCmd);
					}
				}
				makeDirs();
			}

			try {
				objDataSourceClient = (ISOSVfsFileTransfer) objVfs4Source;
				objSourceFileList.objDataSourceClient = objDataSourceClient;
				if (objOptions.passive_mode.value()) {
					objDataSourceClient.passive();
				}

				objDataSourceClient.TransferMode(objOptions.transfer_mode);
				objDataSourceClient.ControlEncoding(objOptions.ControlEncoding.Value());

				long lngPollTimeOut = 0;
				String strPollTimeOut = "";

				if (objOptions.poll_timeout.isDirty()) {
					lngPollTimeOut = objOptions.poll_timeout.value() * 60; // convert to seconds, poll_timeout is defined in minutes
					strPollTimeOut = objOptions.poll_timeout.Value();
				}
				else {
					lngPollTimeOut = objOptions.PollingDuration.getTimeAsSeconds();
					strPollTimeOut = objOptions.PollingDuration.Value();
				}

				long lngCurrentPollingTime = 0;
				long lngPollInterval = objOptions.poll_interval.getTimeAsSeconds();
				long lngNoOfFilesFound = 0;
				String strSourceDir = objOptions.SourceDir.Value();
				String strRemoteDir = objOptions.TargetDir.Value();
				boolean flgRecurseFolders = objOptions.recursive.value();

				if (objOptions.OneOrMoreSingleFilesSpecified()) {
					objSourceFileList.add(getSingleFileNames(), strSourceDir);

					if (objOptions.isFilePollingEnabled() == true) {
						long lngCurrentNumberOfFilesFound = objSourceFileList.List().size();
						while (true) {
							lngNoOfFilesFound = 0;
							// TODO I18N
							if (lngCurrentPollingTime > lngPollTimeOut) { // time exceeded ?
								logger.info(String.format("polling: time '%1$s' is over. polling terminated", strPollTimeOut));
								break;
							}

							for (SOSFileListEntry objItem : objSourceFileList.List()) {
								// long lngFileSize = objItem.getFileSize();
								if (objItem.SourceFileExists()) {
									lngNoOfFilesFound++;
									objItem.setParent(objSourceFileList); // to reinit the file-size
								}
							}
							if (lngNoOfFilesFound == lngCurrentNumberOfFilesFound) {
								break;
							}

							if (objOptions.poll_minfiles.value() > 0 && lngNoOfFilesFound >= objOptions.poll_minfiles.value()) {
								break;
							}

							String strM = String.format("file-polling: going to sleep for %1$d seconds. '%2$d' files found, waiting for '%3$d' files",
									lngPollInterval, lngNoOfFilesFound, lngCurrentNumberOfFilesFound);
							objJSJobUtilities.setStateText(strM);
							logger.info(String.format(strM));
							Thread.sleep(lngPollInterval * 1000); // wait some seconds
							lngCurrentPollingTime += lngPollInterval;
						}
					}
				}
				else {
					String strRegExp4FileNames = objOptions.file_spec.Value();
					objDataSourceClient.changeWorkingDirectory(strSourceDir);
					ISOSVirtualFile objLocFile = objDataSourceClient.getFileHandle(strSourceDir);
					String strM = "";
					if (objOptions.NeedTargetClient() == true) {
						strM = "source directory/file: " + strSourceDir + ", target directory: " + strRemoteDir + ", file regexp: " + strRegExp4FileNames;
					}
					else {
						strM = SOSJADE_D_0200.params(strSourceDir, strRegExp4FileNames);
					}
					logger.debug(strM);
					if (objLocFile.isDirectory() == true) {
						String[] strFileList = null;
						boolean flgDoBreak = false;
						strFileList = objDataSourceClient.getFilelist(strSourceDir, strRegExp4FileNames, 0, flgRecurseFolders);
						long lngCurrentNoOfFilesFound = strFileList.length;
						strM = String.format("%1$d files found for regexp '%2$s'.", lngCurrentNoOfFilesFound, strRegExp4FileNames);
						logger.info(strM);
						objJSJobUtilities.setStateText(strM);
						while (true) {
							if (objOptions.isFilePollingEnabled()) {
								if (lngCurrentPollingTime > lngPollTimeOut) { // time exceeded ?
									logger.info(String.format("file-polling: time '%1$s' is over. polling terminated", strPollTimeOut));
									break;
								}
								if (objOptions.poll_minfiles.isDirty() && lngCurrentNoOfFilesFound >= objOptions.poll_minfiles.value()) { // amount
									flgDoBreak = true;
								}

								lngNoOfFilesFound = lngCurrentNoOfFilesFound;
								strM = String.format("polling: going to sleep for %1$d seconds.", lngPollInterval);
								objJSJobUtilities.setStateText(strM);
								logger.info(String.format(strM));
								Thread.sleep(lngPollInterval * 1000); // wait some seconds
								lngCurrentPollingTime += lngPollInterval;

								strFileList = objDataSourceClient.getFilelist(strSourceDir, strRegExp4FileNames, 0, flgRecurseFolders);
								lngCurrentNoOfFilesFound = strFileList.length;
								strM = String.format("%1$d files found for regexp '%2$s'.", lngCurrentNoOfFilesFound, strRegExp4FileNames);
								logger.info(strM);

								if (lngNoOfFilesFound >= lngCurrentNoOfFilesFound && lngNoOfFilesFound != 0) { // just wait a round for
																												// latecommers
									flgDoBreak = true;
								}
								else {
									flgDoBreak = false;
								}

								if (flgDoBreak == true) {
									break;
								}
							}
							else {
								break;
							}
						} // while
						if (objOptions.MaxFiles.isDirty() == true && strFileList.length > objOptions.MaxFiles.value()) {
							for (int i = 0; i < objOptions.MaxFiles.value(); i++) {
								objSourceFileList.add(strFileList[i]);
							}
						}
						else {
							objSourceFileList.add(strFileList, strSourceDir);
						}
					}
					else { // not a directory, seems to be a filename
						objSourceFileList.add(strSourceDir);
					}
				}

				// TODO in FileListEntry einbauen
				if (checkSteadyStateOfFiles() == false) {

				}

				// TODO in Options setzen anhand der Values in der Liste.
				if (zeroByteFiles == false) {
					// TODO in die SOSFileList einbauen (SkipZeroByteFiles)
					for (SOSFileListEntry objEntry : objSourceFileList.List()) {
						String strSourceFileName = objEntry.SourceFileName();
						ISOSVirtualFile objVF = objDataSourceClient.getFileHandle(strSourceFileName);
						if (objVF.FileExists()) {
							if (objVF.isEmptyFile()) {
								objEntry.setTransferSkipped();
								if (objOptions.remove_files.value()) {
									objVF.delete();
									objEntry.zeroByteCount++;
								}
							}
						}
						else {
							// TODO Datei (nicht mehr) da? Fehler auslsen, weil in Liste enthalten.
						}
					}
				} // (zeroByteFiles == false)

				try {
					if (objOptions.operation.value() == enuJadeOperations.getlist) {
						String strM = SOSJADE_I_0115.get();
						logger.info(strM);
						objJadeReportLogger.info(strM);
					}
					else {
						sendFiles(objSourceFileList);
					}

					if (objOptions.PostTransferCommands.IsNotEmpty()) {
						// TODO Command separator as option
						String[] strA = objOptions.PostTransferCommands.Value().split(";");
						for (String strCmd : strA) {
							strCmd = objOptions.replaceVars(strCmd);
							objDataTargetClient.getHandler().ExecuteCommand(strCmd);
						}
					}

					objDataSourceClient.close();
					if (objOptions.NeedTargetClient() == true) {
						objDataTargetClient.close();
					}

					objSourceFileList.renameAtomicTransferFiles();
					objSourceFileList.DeleteSourceFiles();

					objSourceFileList.EndTransaction();
				}
				catch (JobSchedulerException e) {
					String strM = TRANSACTION_ABORTED.get(e);
					if (objOptions.transactional.value()) {
						logger.error(strM, e);
					}
					else {
						logger.error(strM, e);
					}
					objJadeReportLogger.info(strM);
					objSourceFileList.Rollback();
					throw e;
				}
				finally {
				}

				objSourceFileList.sendMails();
				// TODO in die Klasse SOSFileList oder SOSFtpCommand
				flgReturnCode = printState(flgReturnCode);
				// }
				return flgReturnCode;
			}
			catch (Exception e) {
				setTextProperties();
				objOptions.getTextProperties().put(conKeywordSTATUS, SOSJADE_T_0013.get());
				flgReturnCode = false;
				String strM = SOSJADE_E_0101.get(e.getLocalizedMessage());
				objJadeReportLogger.info(strM);
				// throw new JobSchedulerException(strM, e);
				throw e;
			}
			finally {
				setTextProperties();
			}
		}
		catch (Exception e) {
			setTextProperties();
			objOptions.getTextProperties().put(conKeywordSTATUS, SOSJADE_T_0013.get());
			String strM = SOSJADE_E_0101.get(e);
			objJadeReportLogger.info(strM);
			throw new JobSchedulerException(e);
		}
	}

	// TODO in das DataTarget verlagern? Oder in die FileList? Multithreaded ausfhren?

	public boolean checkSteadyStateOfFiles() {

		@SuppressWarnings("unused")
		final String conMethodName = conClassName + "::checkSteadyStateOfFiles";

		boolean flgAllFilesAreSteady = true;

		if (objOptions.CheckSteadyStateOfFiles.isTrue() && objSourceFileList != null) {
			long lngCheckSteadyStateInterval = 0;
			long lngSteadyCount = 0;

			logger.debug("checking file(s) for steady state");
			for (int i = 0; i < lngSteadyCount; i++) {
				flgAllFilesAreSteady = true;
				for (SOSFileListEntry objFile : objSourceFileList.List()) {
					long lastFileLength = objDataSourceClient.getFileHandle(objFile.SourceFileName()).getFileSize();
					if (lastFileLength != objFile.getFileSize()) {
						flgAllFilesAreSteady = false;
						logger.info(String.format("File '%1$s' changed during checking steady state", objFile.SourceFileName()));
						objFile.setParent(objSourceFileList);
						objFile.setSteady(false);
					}
					else {
						objFile.setSteady(true);
					}
				}

				if (flgAllFilesAreSteady == false) {
					try {
						Thread.sleep(lngCheckSteadyStateInterval);
					}
					catch (InterruptedException e) {
						// TODO Auto-generated catch block
						e.printStackTrace();
					}
				}
				else {
					break;
				}
			}

			if (flgAllFilesAreSteady == false) {
				logger.error("not all files are steady");
				for (SOSFileListEntry objFile : objSourceFileList.List()) {
					if (objFile.isSteady() == false) {
						logger.info(String.format("File '%1$s' is not steady", objFile.SourceFileName()));
					}
				}
				throw new JobSchedulerException("not all files are steady");
			}
		}
		return flgAllFilesAreSteady;
	} // private boolean checkSteadyStateOfFiles

	private void setTextProperties() {

		@SuppressWarnings("unused")
		final String conMethodName = conClassName + "::setTextProperties";

		// TODO das mu beim JobScheduler-Job in die Parameter zurck, aber nur da
		// siehe hierzu das Interface ...

		if (objSourceFileList != null) {
			objOptions.getTextProperties().put(conKeywordSUCCESSFUL_TRANSFERS, String.valueOf(objSourceFileList.SuccessfulTransfers()));
			objOptions.getTextProperties().put(conKeywordFAILED_TRANSFERS, String.valueOf(objSourceFileList.FailedTransfers()));
		}
		// return the number of transferred files
		if (JobSchedulerException.LastErrorMessage.length() <= 0) {
			objOptions.getTextProperties().put(conKeywordSTATUS, SOSJADE_T_0012.get());
		}
		else {
			objOptions.getTextProperties().put(conKeywordSTATUS, SOSJADE_T_0013.get());
		}
		objOptions.getTextProperties().put(conKeyWordLAST_ERROR, JobSchedulerException.LastErrorMessage);

	} // private void setTextProperties

	/**
	 * 
	 * \brief Logout
	 * 
	 * \details
	 *
	 * \return void
	 *
	 */
	public void Logout() {
		try {
			if (objDataTargetClient != null) {
				objDataTargetClient.logout();
				objDataTargetClient.disconnect();
				objDataTargetClient = null;
			}
			if (objDataSourceClient != null) {
				objDataSourceClient.logout();
				objDataSourceClient.disconnect();
				objDataSourceClient = null;
			}
		}
		catch (IOException e) {
			// e.printStackTrace();
		}
	}

	/**
	 * Verzeichnis generieren
	 * @throws Exception
	 */
	private void makeDirs() {
		try {
			boolean cd = true;
			// TODO make_dirs ist nicht dokumentiert.
			// boolean flgMakeDirs = objOptions.make_dirs.Value();
			boolean flgMakeDirs = makeDirs;
			String strTargetDir = objOptions.TargetDir.Value();
			if (flgMakeDirs) {
				if (objDataTargetClient.changeWorkingDirectory(strTargetDir)) {
					cd = true;
				}
				else {
					objDataTargetClient.mkdir(strTargetDir);
					cd = objDataTargetClient.changeWorkingDirectory(strTargetDir);
				}
			}
			else
				if (strTargetDir != null && strTargetDir.length() > 0) {
					cd = objDataTargetClient.changeWorkingDirectory(strTargetDir);
				}
			// TODO alternative_remote_dir, wozu und wie gehen wir damit um?
			if (cd == false && objOptions.alternative_remote_dir.IsNotEmpty()) {// alternative Parameter
				String alternativeRemoteDir = objOptions.alternative_remote_dir.Value();
				logger.debug("..try with alternative parameter [remoteDir=" + alternativeRemoteDir + "]");
				cd = objDataTargetClient.changeWorkingDirectory(alternativeRemoteDir);
				objOptions.TargetDir.Value(alternativeRemoteDir);
			}
		}
		catch (Exception e) {
			throw new JobSchedulerException("..error in makeDirs, cause: " + e);
		}
	} // private void makeDirs()

	// TODO prfen, ob eine Verlagerung in SOSFileList die bessere Lsung ist. Stichwort: Multithreading
	private void sendFiles(final SOSFileList pobjFileList) {
		@SuppressWarnings("unused")
		final String conMethodName = conClassName + "::sendFiles";
		int intMaxParallelTransfers = 0;
		if (objOptions.ConcurrentTransfer.value() == true) {
			intMaxParallelTransfers = objOptions.MaxConcurrentTransfers.value();
		}
		if (objOptions.CumulateFiles.isTrue()) {
			if (objOptions.CumulativeFileDelete.isTrue()) {
//				for (SOSFileListEntry objSourceFile : pobjFileList.List()) {
//					objSourceFile.Options(objOptions);
//					objSourceFile.run();
//				}
//				pobjFileList.List().get(0).getTargetFile(objOptions).delete();
			}
		}
		
		Vector<Thread> objThreads = new Vector<Thread>();
		try {
			/**
			 * if "sendTransferHistory" is activated due to not thread-safe xml-parser
			 * the parallel transfer is omitted.
			 */
			if (intMaxParallelTransfers <= 0 || objOptions.SendTransferHistory.value() == true) {
				for (SOSFileListEntry objSourceFile : pobjFileList.List()) {
					objSourceFile.Options(objOptions);
					objSourceFile.run();
				}
			}
			else {
				int intThreadsRunning = 0;
				for (SOSFileListEntry objListItem : pobjFileList.List()) {
					objListItem.Options(objOptions);
					objListItem.setDataSourceClient(null); // force a new connection
					objListItem.setDataTargetClient(null);
					Thread objT = new Thread(objListItem);
					objT.start();
					synchronized (objThreads) {
						objThreads.add(objT);
					}
					boolean flgThreadsActive = true;
					intThreadsRunning = 0;
					while (flgThreadsActive) {
						flgThreadsActive = false;
						synchronized (objThreads) {
							for (Thread thread : objThreads) {
								if (thread.isAlive()) {
									flgThreadsActive = true;
									intThreadsRunning++;
								}
							}
						}
						try {
							if (flgThreadsActive == true && intThreadsRunning >= intMaxParallelTransfers) {
								Thread.sleep(500); // TODO this should be an option
							}
							if (intThreadsRunning < intMaxParallelTransfers) {
								break;
							}
						}
						catch (InterruptedException e) {
							throw new JobSchedulerException("thread processing interrupted", e);
						}
						catch (Exception e) {
							throw new JobSchedulerException("thread processing interrupted", e);
						}
					}
				}
			}
		}
		catch (JobSchedulerException e) {
			throw e;
		}
	}

	/**
	 * Statt ein Verzeichnis knnen ein oder mehrere - mit semikolen getrennt - dateien zum transferieren angegeben werden
	 * @return
	 */
	private String[] getSingleFileNames() {
		final String conMethodName = conClassName + "::getSingleFileNames";
		Vector<String> filelist = new Vector<String>();

		if (objOptions.file_path.IsNotEmpty() == true) {
			String filePath = objOptions.file_path.Value();
			// String strPS = System.getProperty("file.separator");
			// String strPS = "/"; // standard on FTP/SFTP
			logger.debug(String.format("single file(s) specified : '%1$s'", filePath));
			try {
				String localDir = AddFileSeparator(objOptions.SourceDir.Value().trim());
				// TODO separator-char variable as Option
				String[] strSingleFileNameArray = filePath.split(";");
				for (String strSingleFileName : strSingleFileNameArray) {
					strSingleFileName = strSingleFileName.trim();
					if (strSingleFileName.length() > 0) {
						if (localDir.trim().length() > 0) {
							if (isAPathName(strSingleFileName) == false) {
								/**
								 * Problem with folders, when pgs run on Windows, but has to
								 * create a path for unix-systems.
								 */
								// strT = new File(localDir, strT).getAbsolutePath();
								strSingleFileName = localDir + strSingleFileName;
							}
						}
						filelist.add(strSingleFileName);
					}
				}
			}
			catch (Exception e) {
				String strM = String.format("error in  %1$s", conMethodName);
				logger.error(strM + e);
				throw new JobSchedulerException(strM, e);
			}
		}

		if (objOptions.FileListName.IsNotEmpty() == true) {
			String strFileListName = objOptions.FileListName.Value();
			JSFile objF = new JSFile(strFileListName);
			if (objF.exists() == true) {
				StringBuffer strRec = null;
				while ((strRec = objF.GetLine()) != null) {
					filelist.add(strRec.toString());
				}
				try {
					objF.close();
				}
				catch (IOException e) {
				}
			}
		}

		String[] strA = { "" };
		if (filelist.size() > 0) {
			strA = filelist.toArray(new String[filelist.size()]);
		}
		return strA;
	}

	protected boolean isAPathName(final String pstrFileAndPathName) {
		boolean flgOK = false;
		String lstrPathName = pstrFileAndPathName.replaceAll("\\\\", "/");
		if (lstrPathName.startsWith(".") || lstrPathName.startsWith("..")) { // relative to localdir
			flgOK = false;
		}
		else {
			if (lstrPathName.contains(":/") || lstrPathName.startsWith("/")) {
				flgOK = true;
			}
			else {
				// flgOK = (lstrPathName.contains("/") == true);
			}
		}
		return flgOK;
	}

	private String AddFileSeparator(final String pstrPathName) {
		String strT = pstrPathName;
		int intL = strT.length();
		if (intL > 0) {
			String strL = strT.substring(intL - 1, intL);
			if (strL.equals("\\") || strL.equals("/")) {
			}
			else {
				// strT += File.separator;
				strT += "/";
			}
		}
		return strT;
	}

	/**
	 * Zustand ausgeben. Erst ab log_level 1
	 * @param rc
	 * @return
	 * @throws Exception
	 */
	private boolean printState(boolean rc) throws Exception {
		@SuppressWarnings("unused")
		final String conMethodName = conClassName + "::printState";
		String state = "processing successful ended";

		try {
			int intNoOfFilesTransferred = (int) objSourceFileList.SuccessfulTransfers();
			String strMsg = SOSJADE_E_0100.params(objOptions.file_spec.Value());
			switch (intNoOfFilesTransferred) {
				case 0:
					if (zeroByteCount > 0 && zeroByteFilesRelaxed) {
						state = strMsg;
					}
					else
						if (zeroByteCount > 0 && zeroByteFilesStrict) {
							throw new JobSchedulerException(SOSJADE_I_0104.get());
						}
						else {
							if (objOptions.force_files.value() == true) {
								if (objSourceFileList.List().size() <= 0) {
									objJadeReportLogger.info(strMsg);
									throw new JobSchedulerException(strMsg);
								}
							}
							else {
								state = strMsg;
							}
						}
					rc = objOptions.force_files.value() == false ? true : !zeroByteFilesRelaxed;
					break;
				case 1:
					state = SOSJADE_I_0100.get();
					rc = true;
					break;
				default:
					state = SOSJADE_I_0101.params(intNoOfFilesTransferred);
					rc = true;
					break;
			}
			if (zeroByteCount > 0) {
				state += " " + SOSJADE_I_0102.params(zeroByteCount);
			}
			logger.debug(state);
			objJadeReportLogger.info(state);
			objOptions.getTextProperties().put(conKeywordSTATE, state);
			return rc;
		}
		catch (Exception e) {
			objJadeReportLogger.info(e.getLocalizedMessage());
			throw e;
		}
		finally {
			objOptions.getTextProperties().put(conKeywordSTATE, state);
		}
	}

	/**
	 * @return the state
	 */
	public String getState() {
		String state = (String) objOptions.getTextProperties().get(conKeywordSTATE);
		return state;
	}

	public SOSFileList getFileList() {
		return objSourceFileList;
	}

//	public String replaceVars(String pstrReplaceIn) {
//		@SuppressWarnings("unused")
//		final String conMethodName = conClassName + "::replaceVars";
//		Properties objTextProperties = objOptions.getTextProperties();
//		// Properties arg = getEnvVars(logger);
//		// retVal.putAll(arg);
//
//		try {
//			// zustzliche Parameter generieren
//			objTextProperties.put("date", SOSOptionTime.getCurrentDateAsString());
//			objTextProperties.put("time", SOSOptionTime.getCurrentTimeAsString("hh:mm:ss"));
//			objTextProperties.put("local_user", System.getProperty("user.name"));
//
//			java.net.InetAddress localMachine = java.net.InetAddress.getLocalHost();
//			objTextProperties.put("localhost", localMachine.getHostName());
//			objTextProperties.put("local_host_ip", localMachine.getHostAddress());
//		}
//		catch (Exception uhe) {
//		}
//
//		String strVal = "";
//		String strKey = "";
//
//		// String strParamNameEnclosedInPercentSigns = ".*%\\{([^%]+)\\}.*";
//		String strParamNameEnclosedInPercentSigns = "^.*%\\{([^%]+)\\}.*$";
//		String strParamNameEnclosedInPercentSigns2 = "^.*%\\{([^\\}]+).*$";
//
//		if (isNotNull(pstrReplaceIn)) {
//			try {
//				pstrReplaceIn = pstrReplaceIn.replaceAll("\\n", "\\$\\$N\\$\\$");
//				while (pstrReplaceIn.matches(strParamNameEnclosedInPercentSigns2)) {
//					strKey = pstrReplaceIn.replaceFirst(strParamNameEnclosedInPercentSigns2, "$1");
//					String strPP = "%\\{" + strKey + "\\}";
//					strVal = objOptions.OptionByName(strKey);
//					if (isNotNull(strVal)) {
//						strVal = strVal.replace('\\', '/');
//						pstrReplaceIn = pstrReplaceIn.replaceAll(strPP, Matcher.quoteReplacement(strVal));
//						// logger.debug(Messages.getMsg(JSJ_D_0031, name, strPP, s)); //
//						// "processing job parameter '%1$s': substitute '%2$s' with '%3$s'."
//					}
//					else {
//						strVal = (String) objTextProperties.get(strKey);
//						if (strVal != null) {
//							pstrReplaceIn = pstrReplaceIn.replaceAll(strPP, Matcher.quoteReplacement(strVal));
//						}
//						else {
//							pstrReplaceIn = pstrReplaceIn.replaceAll(strPP, "?" + Matcher.quoteReplacement(strKey) + "?");
//							// logger.warn(Messages.getMsg(JSJ_D_0032, p)); // "variable '%1$s' not found. no substitution done"
//						}
//					}
//				}
//			}
//			catch (Exception e) { // intentionally no error, wrong regexp ?
//			}
//		}
//		pstrReplaceIn = pstrReplaceIn.replaceAll("\\$\\$N\\$\\$", "\n");
//		return pstrReplaceIn;
//	}// private String replaceVars

	/**
	 * 
	 * \brief setJSJobUtilites
	 * 
	 * \details
	 * The JobUtilities are a set of methods used by the SSH-Job or can be used be other, similar, job-
	 * implementations.
	 * 
	 * \return void
	 *
	 * @param pobjJSJobUtilities
	 */
	@Override
	public void setJSJobUtilites(final JSJobUtilities pobjJSJobUtilities) {
		if (pobjJSJobUtilities == null) {
			objJSJobUtilities = this;
		}
		else {
			objJSJobUtilities = pobjJSJobUtilities;
			// objVfs.setJSJobUtilites(pobjJSJobUtilities);
		}
		logger.debug("objJSJobUtilities = " + objJSJobUtilities.getClass().getName());
	}

	/**
	 * 
	 * \brief replaceSchedulerVars
	 * 
	 * \details
	 * Dummy-Method to make sure, that there is always a valid Instance for the JSJobUtilities.
	 * \return 
	 *
	 * @param isWindows
	 * @param pstrString2Modify
	 * @return
	 */
	@Override
	public String replaceSchedulerVars(final boolean isWindows, final String pstrString2Modify) {
		final String conMethodName = conClassName + "::replaceSchedulerVars";
		logger.debug(conMethodName + " as Dummy-call executed. No Instance of JobUtilites specified.");
		return pstrString2Modify;
	}

	/**
	 * 
	 * \brief setJSParam
	 * 
	 * \details
	 * Dummy-Method to make shure, that there is always a valid Instance for the JSJobUtilities.
	 * \return 
	 *
	 * @param pstrKey
	 * @param pstrValue
	 */
	@Override
	public void setJSParam(final String pstrKey, final String pstrValue) {
		final String conMethodName = conClassName + "::setJSParam";
		logger.debug(conMethodName + " as Dummy-call executed. No Instance of JobUtilites specified.");
	}

	@Override
	public void setJSParam(final String pstrKey, final StringBuffer pstrValue) {
		final String conMethodName = conClassName + "::setJSParam";
		logger.debug(conMethodName + " as Dummy-call executed. No Instance of JobUtilites specified.");
	}

	@Override
	public String myReplaceAll(final String pstrSourceString, final String pstrReplaceWhat, final String pstrReplaceWith) {
		String newReplacement = pstrReplaceWith.replaceAll("\\$", "\\\\\\$");
		return pstrSourceString.replaceAll("(?m)" + pstrReplaceWhat, newReplacement);
	}

	@Override
	public String getCurrentNodeName() {
		return "";
	}

	@Override
	public void setStateText(final String pstrStateText) {
		// TODO Auto-generated method stub

	}
}
