/********************************************************* begin of preamble
**
** Copyright (C) 2003-2012 Software- und Organisations-Service GmbH. 
** All rights reserved.
**
** This file may be used under the terms of either the 
**
**   GNU General Public License version 2.0 (GPL)
**
**   as published by the Free Software Foundation
**   http://www.gnu.org/licenses/gpl-2.0.txt and appearing in the file
**   LICENSE.GPL included in the packaging of this file. 
**
** or the
**  
**   Agreement for Purchase and Licensing
**
**   as offered by Software- und Organisations-Service GmbH
**   in the respective terms of supply that ship with this file.
**
** THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS
** IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO,
** THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR
** PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS
** BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
** CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
** SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
** INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
** CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
** ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
** POSSIBILITY OF SUCH DAMAGE.
********************************************************** end of preamble*/
package com.sos.VirtualFileSystem.Options;
import java.util.HashMap;

import org.apache.log4j.Logger;

import com.sos.JSHelper.Annotations.JSOptionClass;
import com.sos.JSHelper.Exceptions.JSExceptionMandatoryOptionMissing;
import com.sos.JSHelper.Listener.JSListener;
import com.sos.JSHelper.Options.SOSOptionBoolean;
import com.sos.i18n.annotation.I18NResourceBundle;

/**
 * \class 		SOSConnection2OptionsAlternate - Options for a connection to an uri (server, site, e.g.)
 *
 * \brief 
 * An Options as a container for the Options super class. 
 * The Option class will hold all the things, which would be otherwise overwritten at a re-creation
 * of the super-class.
 *
 *

 *
 * see \see j:\e\java\development\com.sos.scheduler\src\sos\scheduler\jobdoc\SOSConnection2.xml for (more) details.
 * 
 * \verbatim ;
 * mechanicaly created by JobDocu2OptionsClass.xslt from http://www.sos-berlin.com at 20100917112404 
 * \endverbatim
 */
@JSOptionClass(name = "SOSConnection2OptionsAlternate", description = "Options for a connection to an uri (server, site, e.g.)")
@I18NResourceBundle(baseName = "SOSVirtualFileSystem", defaultLocale = "en")
public class SOSConnection2OptionsAlternate extends SOSConnection2OptionsSuperClass {
	/**
	 * 
	 */
	private static final long	serialVersionUID		= 5924032437179660014L;
	private final String		conClassName			= "SOSConnection2OptionsAlternate";						//$NON-NLS-1$
	@SuppressWarnings("unused")
	private static Logger		logger					= Logger.getLogger(SOSConnection2OptionsAlternate.class);
	private String				strAlternativePrefix	= "";
	public SOSOptionBoolean		ProtocolCommandListener	= new SOSOptionBoolean(this, conClassName + ".Protocol_Command_Listener", // HashMap-Key
																"use (Apache-) ProtocolCommandListener to log all ftp-commands", // Titel
																"false", // InitValue
																"false", // DefaultValue
																false // isMandatory
														);
	
	private SOSConnection2OptionsSuperClass	objAlternativeOptions	= new SOSConnection2OptionsSuperClass();

	/**
	* constructors
	*/
	public SOSConnection2OptionsAlternate() {
	} // public SOSConnection2OptionsAlternate

	public SOSConnection2OptionsAlternate(final String pstrPrefix) {
		strAlternativePrefix = pstrPrefix;
	} // public SOSConnection2OptionsAlternate

	public SOSConnection2OptionsAlternate(final JSListener pobjListener) {
		this();
		this.registerMessageListener(pobjListener);
	} // public SOSConnection2OptionsAlternate

	public SOSConnection2OptionsAlternate(final HashMap<String, String> pobjJSSettings) throws Exception {
		super(pobjJSSettings);
		objAlternativeOptions.setAllOptions(pobjJSSettings, strAlternativePrefix + "alternative_");
		this.addProcessedOptions(objAlternativeOptions.getProcessedOptions());
	} // public SOSConnection2OptionsAlternate (HashMap JSSettings)

	public SOSConnection2OptionsAlternate(final HashMap<String, String> pobjJSSettings, final String pstrPrefix) throws Exception {
		super(pobjJSSettings);
		strAlternativePrefix = pstrPrefix;
		setAllOptions(pobjJSSettings, strAlternativePrefix);
		objAlternativeOptions.setAllOptions(pobjJSSettings, "alternative_" + strAlternativePrefix);
		this.addProcessedOptions(objAlternativeOptions.getProcessedOptions());
	} // public SOSConnection2OptionsAlternate (HashMap JSSettings)

	/**
	 * \brief CheckMandatory - prft alle Muss-Optionen auf Werte
	 *
	 * \details
	 * @throws Exception
	 *
	 * @throws Exception
	 * - wird ausgelst, wenn eine mandatory-Option keinen Wert hat
	 */
	@Override
	// SOSConnection2OptionsAlternateSuperClass
	public void CheckMandatory() {
		try {
			super.CheckMandatory();
		}
		catch (Exception e) {
			throw new JSExceptionMandatoryOptionMissing(e.toString());
		}
	} // public void CheckMandatory ()

	/**
	 * \brief AlternativeOptions
	 *
	 * \details
	 * getter 
	 *
	 * @return the objAlternativeOptions
	 */
	public SOSConnection2OptionsSuperClass Alternatives() {
		return objAlternativeOptions;
	}

	/**
	 * \brief AlternativeOptions - 
	 *
	 * \details
	 * setter 
	 *
	 * @param objAlternativeOptions the value for objAlternativeOptions to set
	 */
	public void AlternativeOptions(final SOSConnection2OptionsSuperClass pobjAlternativeOptions) {
		objAlternativeOptions = pobjAlternativeOptions;
	}
}
