/********************************************************* begin of preamble
**
** Copyright (C) 2003-2012 Software- und Organisations-Service GmbH. 
** All rights reserved.
**
** This file may be used under the terms of either the 
**
**   GNU General Public License version 2.0 (GPL)
**
**   as published by the Free Software Foundation
**   http://www.gnu.org/licenses/gpl-2.0.txt and appearing in the file
**   LICENSE.GPL included in the packaging of this file. 
**
** or the
**  
**   Agreement for Purchase and Licensing
**
**   as offered by Software- und Organisations-Service GmbH
**   in the respective terms of supply that ship with this file.
**
** THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS
** IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO,
** THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR
** PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS
** BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
** CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
** SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
** INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
** CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
** ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
** POSSIBILITY OF SUCH DAMAGE.
********************************************************** end of preamble*/
package com.sos.VirtualFileSystem.SFTP;
import java.io.BufferedReader;
import java.io.File;
import java.io.FileOutputStream;
import java.io.IOException;
import java.io.InputStream;
import java.io.InputStreamReader;
import java.io.OutputStream;
import java.text.DateFormat;
import java.text.SimpleDateFormat;
import java.util.Date;
import java.util.Vector;

import org.apache.log4j.Logger;

import com.jcraft.jsch.Channel;
import com.jcraft.jsch.ChannelExec;
import com.jcraft.jsch.ChannelSftp;
import com.jcraft.jsch.ChannelSftp.LsEntry;
import com.jcraft.jsch.JSch;
import com.jcraft.jsch.Session;
import com.jcraft.jsch.SftpATTRS;
import com.jcraft.jsch.SftpException;
import com.jcraft.jsch.SftpProgressMonitor;
import com.sos.JSHelper.Exceptions.JobSchedulerException;
import com.sos.JSHelper.Options.SOSOptionInFileName;
import com.sos.VirtualFileSystem.Interfaces.ISOSAuthenticationOptions;
import com.sos.VirtualFileSystem.Interfaces.ISOSConnection;
import com.sos.VirtualFileSystem.Interfaces.ISOSVirtualFile;
import com.sos.VirtualFileSystem.Options.SOSConnection2OptionsAlternate;
import com.sos.VirtualFileSystem.Options.SOSConnection2OptionsSuperClass;
import com.sos.VirtualFileSystem.common.SOSVfsTransferBaseClass;
import com.sos.i18n.annotation.I18NResourceBundle;

/**
 * @ressources jsch-0.1.48.jar
 * 
 * @author Robert Ehrlich
 *
 */
@I18NResourceBundle(baseName = "SOSVirtualFileSystem", defaultLocale = "en")
public class SOSVfsSFtpJCraft extends SOSVfsTransferBaseClass {

	@SuppressWarnings("unused")
	private Logger		logger			= Logger.getLogger(SOSVfsSFtpJCraft.class);

	/** ssh connection object */
	private Channel		sshConnection	= null;

	/** ssh session object */
	private Session		sshSession		= null;

	/** SFTP Client **/
	private ChannelSftp	sftpClient		= null;

	private JSch		secureChannel	= null;

	/**
	 * 
	 * \brief SOSVfsSFtpJCraft
	 *
	 * \details
	 *
	 */
	public SOSVfsSFtpJCraft() {
		super();
		
		this.secureChannel = new JSch();
	}

	/**
	 * 
	 * \brief Connect
	 * 
	 * \details
	 *
	 * \return 
	 *
	 * @return
	 */
	@Override
	public ISOSConnection Connect() {
		@SuppressWarnings("unused")
		SOSConnection2OptionsAlternate pConnection2OptionsAlternate = null;
		this.Connect(pConnection2OptionsAlternate);
		return this;
	}

	/**
	 * 
	 * \brief Connect
	 * 
	 * \details
	 *
	 * \return 
	 *
	 * @param pobjConnectionOptions
	 * @return
	 */
	@Override
	public ISOSConnection Connect(SOSConnection2OptionsAlternate pConnection2OptionsAlternate) {
		this.connection2OptionsAlternate = pConnection2OptionsAlternate;

		if (this.connection2OptionsAlternate == null) {
			RaiseException(SOSVfs_E_190.params("connection2OptionsAlternate"));
		}

		this.connect(this.connection2OptionsAlternate.host.Value(), this.connection2OptionsAlternate.port.value());
		return this;
	}

	/**
	 * 
	 * \brief Authenticate
	 * 
	 * \details
	 *
	 * \return 
	 *
	 * @param authenticationOptions
	 * @return
	 */
	@Override
	public ISOSConnection Authenticate(ISOSAuthenticationOptions pAuthenticationOptions) {
		this.authenticationOptions = pAuthenticationOptions;

		try {
			this.doAuthenticate(this.authenticationOptions);
		}
		catch (Exception ex) {
			Exception exx = ex;

			this.disconnect();
			
			if (this.connection2OptionsAlternate != null) {
				SOSConnection2OptionsSuperClass optionsAlternatives = this.connection2OptionsAlternate.Alternatives();

				if (!optionsAlternatives.host.IsEmpty() && !optionsAlternatives.user.IsEmpty()) {
					logINFO(SOSVfs_I_170.params(this.connection2OptionsAlternate.Alternatives().host.Value()));
					try {

						this.host = optionsAlternatives.host.Value();
						this.port = optionsAlternatives.port.value();

						this.doAuthenticate(optionsAlternatives);
						exx = null;
					}
					catch (Exception e) {
						exx = e;
					}
				}
			}

			if (exx != null) {
				RaiseException(exx, SOSVfs_E_168.get());
			}
		}

		return this;
	}

	/**
	 * 
	 * \brief login
	 * 
	 * \details
	 *
	 * \return 
	 *
	 * @param strUserName
	 * @param strPassword
	 */
	@Override
	public void login(String pUserName, String pPassword) {

		try {

			this.userName = pUserName;

			logDEBUG(SOSVfs_D_132.params(this.userName));

			this.createSession(this.userName, this.host, this.port);
			this.sshSession.setPassword(pPassword);
			this.sshSession.connect();

			this.createSftpClient();

			this.reply = "OK";
			logDEBUG(SOSVfs_D_133.params(this.userName));
			this.LogReply();
		}
		catch (Exception e) {
			RaiseException(e, SOSVfs_E_134.params("authentication"));
		}

	} // private boolean login

	/**
	 * 
	 * \brief disconnect
	 * 
	 * \details
	 *
	 * \return 
	 *
	 */
	@Override
	public void disconnect() {
		this.reply = "disconnect OK";

		if (this.sftpClient != null) {
			try {
				this.sftpClient.exit();
				if (this.sftpClient.isConnected()) {
					this.sftpClient.disconnect();
				}
				this.sftpClient = null;
			}
			catch (Exception ex) {
				this.reply = "disconnect: " + ex;
			}
		}

		if (sshConnection != null) {
			try {
				this.sshConnection.disconnect();
				this.sshConnection = null;
			}
			catch (Exception ex) {
				this.reply = "disconnect: " + ex;
			}
		}

		this.logINFO(reply);
	}

	@Override
	public boolean isConnected() {
		return (this.sftpClient != null && this.sftpClient.isConnected());
	}

	/**
	 * Creates a new subdirectory on the FTP server in the current directory .
	 * @param pstrPathName The pathname of the directory to create.
	 * @exception JobSchedulerException
	 */
	@Override
	public void mkdir(String path) {
		try {
			this.reply = "";
			String strPath = "";
			String[] pathArray = path.split("/");
			for (String strSubFolder : pathArray) {
				if (strSubFolder.trim().length() > 0) {
					strPath += strSubFolder + "/";
					logDEBUG(HostID(SOSVfs_D_179.params("mkdir", strPath)));
					
					this.getClient().mkdir(strPath);
					this.getClient().chmod(484, strPath);
					this.reply = "mkdir OK";
					logDEBUG(HostID(SOSVfs_D_181.params("mkdir", strPath, getReplyString())));
				}
			}
			// DoCD(strCurrentDir);
			logINFO(HostID(SOSVfs_D_181.params("mkdir", path, getReplyString())));
		}
		catch (Exception e) {
			this.reply = e.toString();
			RaiseException(e, SOSVfs_E_134.params("[mkdir]"));
		}
	}

	/**
	 * Removes a directory on the FTP server (if empty).
	 * @param path The pathname of the directory to remove.
	 * @exception JobSchedulerException
	 */
	@Override
	public void rmdir(String path) {
		try {
			this.reply = "rmdir OK";
			String[] pathArray = path.split("/");
			for (int i = pathArray.length; i > 0; i--) {
				String strT = "";
				for (int j = 0; j < i; j++) {
					strT += pathArray[j] + "/";
				}
				logDEBUG(HostID(SOSVfs_D_179.params("rmdir", strT)));
				this.getClient().rmdir(strT);

				this.reply = "rmdir OK";
				logDEBUG(HostID(SOSVfs_D_181.params("rmdir", strT, getReplyString())));
			}
			logINFO(HostID(SOSVfs_D_181.params("rmdir", path, getReplyString())));
		}
		catch (Exception e) {
			this.reply = e.toString();
			RaiseException(e, SOSVfs_E_134.params("[rmdir]"));
		}
	}

	/**
	 * Checks if file is a directory
	 * 
	 * @param filename
	 * @return true, if filename is a directory
	 */
	@Override
	public boolean isDirectory(String filename) {
		try {
			return this.getClient().stat(filename).isDir();
		}
		catch (Exception e) {
		}

		return false;
	}

	/**
	 * 
	 * \brief listNames
	 * 
	 * \details
	 *
	 * \return 
	 *
	 * @param path
	 * @return
	 * @throws IOException
	 */
	@Override
	public String[] listNames(String path) throws IOException {
		path = resolvePathname(path);
		try {
			if (path.length() == 0) {
				path = ".";
			}
			if (!this.fileExists(path)) {
				return null;
			}
			if (!isDirectory(path)) {
				this.reply = "ls OK";
				return new String[] { path };
			}
			Vector lsResult = this.getClient().ls(path);
			String[] result = new String[lsResult.size()];

			for (int i = 0; i < lsResult.size(); i++) {
				LsEntry entry = (LsEntry) lsResult.get(i);
				String sep = (path.endsWith("/")) ? "" : "/";
				result[i] = path + sep + entry.getFilename();
			}
			this.reply = "ls OK";
			return result;
		}
		catch (Exception e) {
			this.reply = e.toString();
			return null;
		}
	}

	/**
	 * return the size of remote-file on the remote machine on success, otherwise -1
	 * @param remoteFile the file on remote machine
	 * @return the size of remote-file on remote machine
	 */
	@Override
	public long size(String remoteFile) throws Exception {
		remoteFile = this.resolvePathname(remoteFile);
		long size = -1;

		SftpATTRS objAttr;
		try {
			objAttr = this.getClient().stat(remoteFile);
			if (objAttr != null) {
				size = objAttr.getSize();
			}
		}
		catch (SftpException e) {
			// e.printStackTrace();
		}

		return size;
	}

	/**
	 * 
	 * \brief getFile
	 * 
	 * \details
	 *
	 * \return 
	 *
	 * @param remoteFile
	 * @param localFile
	 * @param append
	 * @return
	 */
	@Override
	public long getFile(String remoteFile, String localFile, boolean append) {
		String sourceLocation = this.resolvePathname(remoteFile);
		File transferFile = null;
		long remoteFileSize = -1;
		FileOutputStream fos = null;

		try {
			remoteFileSize = this.size(remoteFile);

			fos = new FileOutputStream(localFile, append);

			this.getClient().get(sourceLocation, fos);

			fos.flush();
			fos.close();
			fos = null;

			transferFile = new File(localFile);

			if (!append) {
				if (remoteFileSize > 0 && remoteFileSize != transferFile.length()) {
					throw new JobSchedulerException(SOSVfs_E_162.params(remoteFileSize, transferFile.length()));
				}
			}

			remoteFileSize = transferFile.length();
			this.reply = "get OK";
			logINFO(HostID(SOSVfs_I_182.params("getFile", sourceLocation, localFile, getReplyString())));
		}
		catch (Exception ex) {
			this.reply = ex.toString();
			RaiseException(ex, SOSVfs_E_184.params("getFile", sourceLocation, localFile));
		}
		finally {
			if (fos != null) {
				try {
					fos.close();
				}
				catch (Exception ex) {
				}
			}
		}

		return remoteFileSize;
	}

	/**
	 * Stores a file on the server using the given name.
	 * 
	 * @param localFile The name of the local file.
	 * @param remoteFile The name of the remote file.
	 * @return file size
	 * 
	 * @exception Exception
	 * @see #put( String, String )
	 */
	@Override
	// ISOSVfsFileTransfer
	public long putFile(String localFile, String remoteFile) {
		long size = 0;
		try {
			// mit Progress Monitor
			// SftpProgressMonitor monitor = new ProgressMonitor();
			// this.getClient().put(localFile, this.resolvePathname(remoteFile),monitor,ChannelSftp.OVERWRITE);

			this.getClient().put(localFile, this.resolvePathname(remoteFile), ChannelSftp.OVERWRITE);

			this.reply = "put OK";
			logINFO(HostID(SOSVfs_I_183.params("putFile", localFile, remoteFile, getReplyString())));
			return this.size(remoteFile);
		}
		catch (Exception e) {
			this.reply = e.toString();
			RaiseException(e, SOSVfs_E_185.params("putFile()", localFile, remoteFile));
		}

		return size;
	}

	/**
	 * Deletes a file on the FTP server.
	 * @param The path of the file to be deleted.
	 * @return True if successfully completed, false if not.
	 * @throws RunTime error occurs while either sending a
	 * command to the server or receiving a reply from the server.
	 */
	@Override
	public void delete(String path) {
		try {
			if (this.isDirectory(path)) {
				throw new JobSchedulerException(SOSVfs_E_186.params(path));
			}

			this.getClient().rm(path);
		}
		catch (Exception ex) {
			this.reply = ex.toString();
			RaiseException(ex, SOSVfs_E_187.params("delete", path));
		}

		this.reply = "rm OK";
		logINFO(HostID(SOSVfs_D_181.params("delete", path, getReplyString())));
	}

	/**
	 * 
	 * \brief rename
	 * 
	 * \details
	 *
	 * \return 
	 *
	 * @param from
	 * @param to
	 */
	@Override
	public void rename(String from, String to) {
		from = this.resolvePathname(from);
		to = this.resolvePathname(to);
		try {
			this.getClient().rename(from, to);
		}
		catch (Exception e) {
			this.reply = e.toString();
			RaiseException(e, SOSVfs_E_188.params("rename", from, to));
		}

		this.reply = "mv OK";
		logINFO(HostID(SOSVfs_I_189.params(from, to, getReplyString())));
	}

	/**
	 * 
	 * \brief ExecuteCommand
	 * 
	 * \details
	 *
	 * \return 
	 *
	 * @param strCmd
	 */
	@Override
	public void ExecuteCommand(String cmd) {

		final String strEndOfLine = System.getProperty("line.separator");

		ChannelExec channelExec = null;
		Integer exitCode = null;
		InputStream out = null;
		InputStream err = null;
		BufferedReader errReader = null;

		try {
			if (this.sshSession == null) {
				// sshSession = sshConnection.openSession();
				// TODO create Session ??
				throw new JobSchedulerException(SOSVfs_E_190.params("sshSession"));
			}

			channelExec = (ChannelExec) this.sshSession.openChannel("exec");
			channelExec.setCommand(cmd);
			channelExec.setInputStream(null);
			channelExec.setErrStream(null);

			out = channelExec.getInputStream();
			err = channelExec.getErrStream();

			channelExec.connect();

			logDEBUG(SOSVfs_D_163.params("stdout", cmd));
			StringBuffer outContent = new StringBuffer();
			byte[] tmp = new byte[1024];

			while (true) {
				while (out.available() > 0) {
					int i = out.read(tmp, 0, 1024);
					if (i < 0) {
						break;
					}

					// output line
					outContent.append(new String(tmp, 0, i) + strEndOfLine);
				}
				if (channelExec.isClosed()) {
					exitCode = channelExec.getExitStatus();
					break;
				}
				try {
					Thread.sleep(1000);
				}
				catch (Exception ee) {
				}
			}
			logDEBUG(outContent);

			logDEBUG(SOSVfs_D_163.params("stderr", cmd));
			errReader = new BufferedReader(new InputStreamReader(err));
			StringBuffer errContent = new StringBuffer();
			while (true) {
				String line = errReader.readLine();
				if (line == null) {
					break;
				}
				errContent.append(line + strEndOfLine);
			}
			logDEBUG(errContent);

			if (exitCode != null) {
				if (!exitCode.equals(new Integer(0))) {
					throw new JobSchedulerException(SOSVfs_E_164.params(exitCode));
				}
			}

			this.reply = "OK";
			logINFO(HostID(SOSVfs_I_192.params(getReplyString())));
		}
		catch (Exception ex) {
			this.reply = ex.toString();
			RaiseException(ex, SOSVfs_E_134.params("ExecuteCommand"));
		}
		finally {
			if (out != null) {
				try {
					out.close();
				}
				catch (Exception e) {
				}
			}
			if (errReader != null) {
				try {
					errReader.close();
				}
				catch (Exception e) {
				}
			}
			if (err != null) {
				try {
					err.close();
				}
				catch (Exception e) {
				}
			}
			if (channelExec != null) {
				try {
					channelExec.disconnect();
				}
				catch (Exception e) {
				}
			}
		}
	}

	/**
	 * 
	 * \brief getInputStream
	 * 
	 * \details
	 *
	 * \return 
	 *
	 * @param fileName
	 * @return
	 */
	@Override
	public InputStream getInputStream(String fileName) {
		try {
			return this.getClient().get(fileName);
		}
		catch (Exception ex) {
			RaiseException(ex, SOSVfs_E_193.params("getInputStream()", fileName));
			return null;
		}
	}

	/**
	 * 
	 * \brief getOutputStream
	 * 
	 * \details
	 *
	 * \return 
	 *
	 * @param fileName
	 * @return
	 */
	@Override
	public OutputStream getOutputStream(String fileName) {
		try {
			return this.getClient().put(fileName);
		}
		catch (Exception ex) {
			RaiseException(ex, SOSVfs_E_193.params("getOutputStream()", fileName));
			return null;
		}
	}

	@Override
	public boolean changeWorkingDirectory(String pathname) {
		try {
			pathname = this.resolvePathname(pathname);
			
			if (!this.fileExists(pathname)) {
				this.reply = "\"" + pathname + "\" doesn't exist.";
				return false;
			}
			if (!this.isDirectory(pathname)) {
				this.reply = "\"" + pathname + "\" is not a directory.";
				return false;
			}
			
			this.getClient().cd(pathname);
			this.reply = "cwd OK";
		}
		catch (Exception ex) {
			RaiseException(ex, SOSVfs_E_193.params("cwd", pathname));
		}
		finally {
			logDEBUG(SOSVfs_D_194.params(pathname, getReplyString()));
		}
		return true;
	}

	@Override
	public ISOSVirtualFile getFileHandle(String fileName) {
		fileName = adjustFileSeparator(fileName);
		ISOSVirtualFile file = new SOSVfsSFtpFileJCraft(fileName);
		file.setHandler(this);

		logDEBUG(SOSVfs_D_196.params(fileName));

		return file;
	}

	/**
	 * 
	 * \brief getModificationTime
	 * 
	 * \details
	 *
	 * \return 
	 *
	 * @param path
	 * @return
	 */
	@Override
	public String getModificationTime(String path) {
		String dateTime = null;
	
		try {
			SftpATTRS objAttr = this.getClient().stat(path);
			if (objAttr != null) {
				int mt = objAttr.getMTime();
				DateFormat df = new SimpleDateFormat("yyyy-MM-dd HH:mm:ss");
				dateTime = df.format(new Date(mt));
			}
		}
		catch (SftpException e) {
			// e.printStackTrace();
		}
		return dateTime;
	}
	
	
	/**
	 * 
	 * \brief fileExists
	 * 
	 * \details
	 *
	 * \return 
	 *
	 * @param filename
	 * @return
	 */
	@Override
	protected boolean fileExists(String filename) {

		try {
			SftpATTRS attributes = this.getClient().stat(filename);

			if (attributes != null) {
				return (!attributes.isLink() || attributes.isDir());
			}
			else {
				return false;
			}
		}
		catch (Exception e) {
			return false;
		}
	}

	protected String getCurrentPath() {
		String path = null;

		try {
			path = this.getClient().pwd();
			logDEBUG(HostID(SOSVfs_D_195.params(path)));
			LogReply();
		}
		catch (Exception e) {
			RaiseException(e, SOSVfs_E_134.params("getCurrentPath"));
		}

		return path;
	}

	/**
	 * 
	 * \brief doAuthenticate
	 * 
	 * \details
	 *
	 * \return ISOSConnection
	 *
	 * @param authenticationOptions
	 * @return
	 * @throws Exception
	 */
	private ISOSConnection doAuthenticate(ISOSAuthenticationOptions pAuthenticationOptions) throws Exception {

		this.authenticationOptions = pAuthenticationOptions;

		this.userName = this.authenticationOptions.getUser().Value();
		String password = this.authenticationOptions.getPassword().Value();
		logDEBUG(SOSVfs_D_132.params(this.userName));

		this.createSession(this.userName, this.host, this.port);

		if (this.authenticationOptions.getAuth_method().isPublicKey()) {
			logDEBUG(SOSVfs_D_165.params("userid", "publickey"));

			SOSOptionInFileName authenticationFile = this.authenticationOptions.getAuth_file();
			authenticationFile.CheckMandatory(true);
			if (authenticationFile.IsNotEmpty()) {
				this.secureChannel.addIdentity(authenticationFile.JSFile().getPath());
			}
		}
		else {
			if (this.authenticationOptions.getAuth_method().isPassword()) {
				logDEBUG(SOSVfs_D_165.params("userid", "password"));
				this.sshSession.setPassword(password);
			}
			else {
				throw new JobSchedulerException(SOSVfs_E_166.params(this.authenticationOptions.getAuth_method().Value()));
			}
		}

		try {
			this.sshSession.connect();
			this.createSftpClient();
		}
		catch (Exception ex) {
			throw new JobSchedulerException(SOSVfs_E_167.params(this.authenticationOptions.getAuth_method().Value(),
					this.authenticationOptions.getAuth_file().Value()));
		}

		this.reply = "OK";
		logDEBUG(SOSVfs_D_133.params(this.userName));
		this.LogReply();

		return this;
	}

	private ChannelSftp getClient() {
		if (this.sftpClient == null) {
			try {
				if (this.sshConnection == null) {
					RaiseException(SOSVfs_E_190.params("sshConnection Object"));
				}

				this.sftpClient = (ChannelSftp) this.sshConnection;
			}
			catch (Exception e) {
				RaiseException(e, SOSVfs_E_196.get());
			}
		}
		return this.sftpClient;
	}

	/**
	 * 
	 * \brief connect
	 * 
	 * \details
	 *
	 * \return void
	 *
	 * @param phost
	 * @param pport
	 */
	private void connect(String phost, int pport) {

		this.host = phost;
		this.port = pport;

		logDEBUG(SOSVfs_D_0101.params(host, port));

		if (this.isConnected() == false) {
			this.LogReply();
		}
		else {
			logWARN(SOSVfs_D_0103.params(host, port));
		}
	}

	/**
	 * 
	 * \brief createSession
	 * 
	 * \details
	 *
	 * \return void
	 *
	 * @param puser
	 * @param phost
	 * @param pport
	 * @throws Exception
	 */
	private void createSession(String puser, String phost, int pport) throws Exception {

		if (this.secureChannel == null) {
			throw new JobSchedulerException(SOSVfs_E_190.params("secureChannel"));
		}
		this.sshSession = this.secureChannel.getSession(puser, phost, pport);

		java.util.Properties config = new java.util.Properties();
		// TODO "StrictHostKeyChecking" ber eine option steuern
		config.put("StrictHostKeyChecking", "no");
		this.sshSession.setConfig(config);
	}

	/**
	 * 
	 * \brief createObjFTPClient
	 * 
	 * \details
	 *
	 * \return void
	 *
	 * @throws Exception
	 */
	private void createSftpClient() throws Exception {

		if (this.sshSession == null) {
			throw new JobSchedulerException(SOSVfs_E_190.params("sshSession"));
		}

		this.sshConnection = this.sshSession.openChannel("sftp");
		this.sshConnection.connect();

		this.sftpClient = (ChannelSftp) this.sshConnection;
	}

	/**
	 * 
	 * @author re
	 *
	 * Progress monitor (sftp operations GET & PUT)
	 *
	 */
	private static class ProgressMonitor implements SftpProgressMonitor {
		long	totalSize			= 0;
		long	transmittedSize		= 0;
		long	transmittedPercent	= -1;

		public void init(int operation, String src, String dest, long size) {
			this.totalSize = size;
			this.transmittedSize = 0;
			this.transmittedPercent = -1;
		}

		public boolean count(long transmitted) {

			System.out.println("Completed " + this.transmittedSize + "(" + this.transmittedPercent + "%) out of " + this.totalSize + ".");

			this.transmittedSize += transmitted;

			long percent = this.transmittedSize * 100 / this.totalSize;

			if (this.transmittedPercent >= percent) {
				return true;
			}

			this.transmittedPercent = percent;

			return true;
		}

		public void end() {

			System.out.println("END " + this.transmittedSize + "(" + this.transmittedPercent + "%) out of " + this.totalSize + ".");
		}
	}
}
