/********************************************************* begin of preamble
**
** Copyright (C) 2003-2012 Software- und Organisations-Service GmbH. 
** All rights reserved.
**
** This file may be used under the terms of either the 
**
**   GNU General Public License version 2.0 (GPL)
**
**   as published by the Free Software Foundation
**   http://www.gnu.org/licenses/gpl-2.0.txt and appearing in the file
**   LICENSE.GPL included in the packaging of this file. 
**
** or the
**  
**   Agreement for Purchase and Licensing
**
**   as offered by Software- und Organisations-Service GmbH
**   in the respective terms of supply that ship with this file.
**
** THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS
** IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO,
** THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR
** PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS
** BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
** CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
** SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
** INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
** CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
** ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
** POSSIBILITY OF SUCH DAMAGE.
********************************************************** end of preamble*/
package com.sos.VirtualFileSystem.WebDAV;
import java.io.ByteArrayOutputStream;
import java.io.File;
import java.io.IOException;
import java.io.InputStream;
import java.io.OutputStream;
import java.text.DateFormat;
import java.text.SimpleDateFormat;
import java.util.Date;

import org.apache.commons.httpclient.HttpException;
import org.apache.commons.httpclient.HttpURL;
import org.apache.commons.httpclient.HttpsURL;
import org.apache.log4j.Logger;
import org.apache.webdav.lib.WebdavResource;

import com.sos.JSHelper.Exceptions.JobSchedulerException;
import com.sos.VirtualFileSystem.Interfaces.ISOSAuthenticationOptions;
import com.sos.VirtualFileSystem.Interfaces.ISOSConnection;
import com.sos.VirtualFileSystem.Interfaces.ISOSVirtualFile;
import com.sos.VirtualFileSystem.Options.SOSConnection2OptionsAlternate;
import com.sos.VirtualFileSystem.Options.SOSConnection2OptionsSuperClass;
import com.sos.VirtualFileSystem.common.SOSVfsTransferBaseClass;
import com.sos.i18n.annotation.I18NResourceBundle;

/**
 * @ressources webdavclient4j-core-0.92.jar,jdom-1.0.jar
 * 
 * @author Robert Ehrlich
 *
 */
@I18NResourceBundle(baseName = "SOSVirtualFileSystem", defaultLocale = "en")
public class SOSVfsWebDAV extends SOSVfsTransferBaseClass {

	@SuppressWarnings("unused")
	private Logger			logger		= Logger.getLogger(SOSVfsWebDAV.class);

	private WebdavResource	davClient	= null;
	private HttpURL			rootUrl		= null;

	/**
	 * 
	 * \brief SOSVfsWebDAV
	 *
	 * \details
	 *
	 */
	public SOSVfsWebDAV() {
		super();
	}

	/**
	 * 
	 * \brief Connect
	 * 
	 * \details
	 *
	 * \return 
	 *
	 * @return
	 */
	@Override
	public ISOSConnection Connect() {
		@SuppressWarnings("unused")
		SOSConnection2OptionsAlternate pConnection2OptionsAlternate = null;
		this.Connect(pConnection2OptionsAlternate);
		return this;
	}

	/**
	 * 
	 * \brief Connect
	 * 
	 * \details
	 *
	 * \return 
	 *
	 * @param pobjConnectionOptions
	 * @return
	 */
	@Override
	public ISOSConnection Connect(SOSConnection2OptionsAlternate pConnection2OptionsAlternate) {
		this.connection2OptionsAlternate = pConnection2OptionsAlternate;

		if (this.connection2OptionsAlternate == null) {
			RaiseException(SOSVfs_E_190.params("connection2OptionsAlternate"));
		}

		this.connect(this.connection2OptionsAlternate.host.Value(), this.connection2OptionsAlternate.port.value());
		return this;
	}

	/**
	 * 
	 * \brief Authenticate
	 * 
	 * \details
	 *
	 * \return 
	 *
	 * @param pAuthenticationOptions
	 * @return
	 */
	@Override
	public ISOSConnection Authenticate(ISOSAuthenticationOptions pAuthenticationOptions) {
		this.authenticationOptions = pAuthenticationOptions;

		try {
			this.doAuthenticate(this.authenticationOptions);
		}
		catch (Exception ex) {
			Exception exx = ex;
			
			this.disconnect();

			if (this.connection2OptionsAlternate != null) {
				SOSConnection2OptionsSuperClass optionsAlternatives = this.connection2OptionsAlternate.Alternatives();
				if (!optionsAlternatives.host.IsEmpty() && !optionsAlternatives.user.IsEmpty()) {
					logINFO(SOSVfs_I_170.params(this.connection2OptionsAlternate.Alternatives().host.Value()));
					try {

						this.host = optionsAlternatives.host.Value();
						this.port = optionsAlternatives.port.value();

						this.doAuthenticate(optionsAlternatives);
						exx = null;
					}
					catch (Exception e) {
						exx = e;
					}
				}
			}

			if (exx != null) {
				RaiseException(exx, SOSVfs_E_168.get());
			}
		}

		return this;
	}

	/**
	 * 
	 * \brief login
	 * 
	 * \details
	 *
	 * \return 
	 *
	 * @param pUserName
	 * @param pPassword
	 */
	@Override
	public void login(String pUserName, String pPassword) {

		try {

			this.userName = pUserName;

			logDEBUG(SOSVfs_D_132.params(this.userName));

			this.setRootHttpURL(this.userName, pPassword, this.host, this.port);
			this.davClient = new WebdavResource(this.rootUrl);

			if (!this.davClient.exists()) {
				throw new Exception("not connected " + this.davClient.getStatusMessage());
			}

			this.reply = "OK";
			logDEBUG(SOSVfs_D_133.params(this.userName));
			this.LogReply();
		}
		catch (Exception e) {
			RaiseException(e, SOSVfs_E_134.params("authentication"));
		}

	} // private boolean login

	/**
	 * 
	 * \brief disconnect
	 * 
	 * \details
	 *
	 * \return 
	 *
	 */
	@Override
	public void disconnect() {
		this.reply = "disconnect OK";

		if (this.davClient != null) {
			try {
				this.davClient.close();
				this.davClient = null;
			}
			catch (Exception ex) {
				this.reply = "disconnect: " + ex;
			}
		}

		this.logINFO(reply);
	}

	/**
	 * 
	 * \brief isConnected
	 * 
	 * \details
	 *
	 * \return 
	 *
	 * @return
	 */
	@Override
	public boolean isConnected() {
		return this.davClient != null;
	}

	/**
	 * Creates a new subdirectory on the FTP server in the current directory .
	 * @param path The pathname of the directory to create.
	 * @exception JobSchedulerException
	 */
	@Override
	public void mkdir(String path) {
		try {
			this.reply = "";
			String strPath = "";
			String[] pathArray = path.split("/");
			for (String strSubFolder : pathArray) {
				if (strSubFolder.trim().length() > 0) {
					strPath += strSubFolder + "/";
					strPath = this.normalizePath(strPath);

					logDEBUG(HostID(SOSVfs_D_179.params("mkdir", strPath)));

					if (this.fileExists(strPath)) {
						continue;
					}

					if (this.davClient.mkcolMethod(strPath)) {
						this.reply = "mkdir OK";
						logDEBUG(HostID(SOSVfs_D_181.params("mkdir", strPath, getReplyString())));
					}
					else {
						throw new Exception(this.davClient.getStatusMessage());
					}
				}
			}

			// DoCD(strCurrentDir);
			logINFO(HostID(SOSVfs_D_181.params("mkdir", path, getReplyString())));
		}
		catch (Exception e) {
			this.reply = e.toString();
			RaiseException(e, SOSVfs_E_134.params("[mkdir]"));
		}
	}

	/**
	 * Removes a directory on the FTP server (if empty).
	 * @param path The pathname of the directory to remove.
	 * @exception JobSchedulerException
	 */
	@Override
	public void rmdir(String path) {
		try {
			this.reply = "rmdir OK";

			path = this.normalizePath(path);

			if (this.davClient.deleteMethod(path)) {
				this.reply = "rmdir OK";
				logDEBUG(HostID(SOSVfs_D_181.params("rmdir", path, getReplyString())));
			}
			else {
				throw new Exception(this.davClient.getStatusMessage());
			}

			logINFO(HostID(SOSVfs_D_181.params("rmdir", path, getReplyString())));
		}
		catch (Exception e) {
			this.reply = e.toString();
			RaiseException(e, SOSVfs_E_134.params("[rmdir]"));
		}
	}

	/**
	 * Checks if file is a directory
	 * 
	 * @param path
	 * @return true, if filename is a directory
	 */
	@Override
	public boolean isDirectory(String path) {
		WebdavResource res = null;
		try {

			res = this.getResource(path);

			return res.isCollection();
		}
		catch (Exception e) {
		}
		finally {
			try {
				if (res != null) {
					res.close();
				}
			}
			catch (Exception ex) {
			}
		}
		return false;
	}

	/**
	 * 
	 * \brief listNames
	 * 
	 * \details
	 *
	 * \return 
	 *
	 * @param path
	 * @return
	 * @throws IOException
	 */
	@Override
	public String[] listNames(String path) throws IOException {
		WebdavResource res = null;
		try {
			if (path.length() == 0) {
				path = "/";
			}
			if (!this.fileExists(path)) {
				return null;
			}

			if (!this.isDirectory(path)) {
				this.reply = "ls OK";
				return new String[] { path };
			}

			res = this.getResource(path);

			WebdavResource[] lsResult = res.listWebdavResources();
			String[] result = new String[lsResult.length];
			for (int i = 0; i < lsResult.length; i++) {
				WebdavResource entry = lsResult[i];
				result[i] = entry.getPath();
			}
			this.reply = "ls OK";
			return result;
		}
		catch (Exception e) {
			this.reply = e.toString();
			return null;
		}
		finally {
			if (res != null) {
				try {
					res.close();
				}
				catch (Exception ex) {
				}
			}
		}
	}

	/**
	 * return the size of remote-file on the remote machine on success, otherwise -1
	 * @param path the file on remote machine
	 * @return the size of remote-file on remote machine
	 */
	@Override
	public long size(String path) throws Exception {
		long size = -1;
		WebdavResource res = null;
		try {
			res = this.getResource(path);
			if (res.exists()) {
				size = res.getGetContentLength();
			}
		}
		catch (Exception e) {
			e.printStackTrace(System.err);
			throw new Exception(SOSVfs_E_161.params("checking size", e));
		}
		finally {
			if (res != null) {
				try {
					res.close();
				}
				catch (Exception ex) {
				}
			}
		}

		return size;
	}

	/**
	 * 
	 * \brief getFile
	 * 
	 * \details
	 *
	 * \return 
	 *
	 * @param remoteFile
	 * @param localFile
	 * @param append
	 * @return
	 */
	@Override
	public long getFile(String remoteFile, String localFile, boolean append) {
		String sourceLocation = this.resolvePathname(remoteFile);
		File transferFile = null;
		long remoteFileSize = -1;
		File file = null;
		WebdavResource res = null;
		try {
			remoteFile = this.normalizePath(remoteFile);

			//remoteFileSize = this.size(remoteFile);

			// TODO append muss noch bercksichtigt werden
			file = new File(localFile);

			res = this.getResource(remoteFile);
			if (!res.exists()) {
				throw new Exception("remoteFile not found");
			}

			remoteFileSize = res.getGetContentLength();

			
			if (res.getMethod(sourceLocation, file)) {
				transferFile = new File(localFile);

				if (!append) {
					if (remoteFileSize > 0 && remoteFileSize != transferFile.length()) {
						throw new JobSchedulerException(SOSVfs_E_162.params(remoteFileSize, transferFile.length()));
					}
				}

				remoteFileSize = transferFile.length();
				this.reply = "get OK";
				logINFO(HostID(SOSVfs_I_182.params("getFile", sourceLocation, localFile, getReplyString())));
			}
			else {
				throw new Exception(res.getStatusMessage());
			}
		}
		catch (Exception ex) {
			this.reply = ex.toString();
			RaiseException(ex, SOSVfs_E_184.params("getFile", sourceLocation, localFile));
		}
		finally {
			try {
				if (res != null) {
					res.close();
				}
			}
			catch (Exception e) {
			}
		}

		return remoteFileSize;
	}

	/**
	 * Stores a file on the server using the given name.
	 * 
	 * @param localFile The name of the local file.
	 * @param remoteFile The name of the remote file.
	 * @return file size
	 * 
	 * @exception Exception
	 * @see #put( String, String )
	 */
	@Override
	// ISOSVfsFileTransfer
	public long putFile(String localFile, String remoteFile) {
		long size = 0;
		try {
			remoteFile = this.normalizePath(remoteFile);

			if (this.davClient.putMethod(remoteFile, new File(localFile))) {
				this.reply = "put OK";
				logINFO(HostID(SOSVfs_I_183.params("putFile", localFile, remoteFile, getReplyString())));
				return this.size(remoteFile);
			}
			else {
				throw new Exception(this.davClient.getStatusMessage());
			}
		}
		catch (Exception e) {
			this.reply = e.toString();
			RaiseException(e, SOSVfs_E_185.params("putFile()", localFile, remoteFile));
		}

		return size;
	}

	/**
	 * Deletes a file on the FTP server.
	 * @param The path of the file to be deleted.
	 * @return True if successfully completed, false if not.
	 * @throws RunTime error occurs while either sending a
	 * command to the server or receiving a reply from the server.
	 */
	@Override
	public void delete(String path) {
		try {
			path = this.normalizePath(path);

			if (this.isDirectory(path)) {
				throw new JobSchedulerException(SOSVfs_E_186.params(path));
			}

			if (!this.davClient.deleteMethod(path)) {
				throw new Exception(this.davClient.getStatusMessage());
			}
		}
		catch (Exception ex) {
			this.reply = ex.toString();
			RaiseException(ex, SOSVfs_E_187.params("delete", path));
		}

		this.reply = "rm OK";
		logINFO(HostID(SOSVfs_D_181.params("delete", path, getReplyString())));
	}

	/**
	 * 
	 * \brief rename
	 * 
	 * \details
	 *
	 * \return 
	 *
	 * @param from
	 * @param to
	 */
	@Override
	public void rename(String from, String to) {
		from = this.resolvePathname(from);
		to = this.resolvePathname(to);
		try {
			from = this.normalizePath(from);
			to = this.normalizePath(to);

			if (!this.davClient.moveMethod(from, to)) {
				throw new Exception(this.davClient.getStatusMessage());
			}
		}
		catch (Exception e) {
			this.reply = e.toString();
			RaiseException(e, SOSVfs_E_188.params("rename", from, to));
		}

		this.reply = "mv OK";
		logINFO(HostID(SOSVfs_I_189.params(from, to, getReplyString())));
	}

	/**
	 * 
	 * \brief ExecuteCommand
	 * 
	 * \details
	 *
	 * \return 
	 *
	 * @param strCmd
	 */
	@Override
	public void ExecuteCommand(String cmd) {
		logDEBUG("not implemented yet");
	}

	/**
	 * 
	 * \brief getInputStream
	 * 
	 * \details
	 *
	 * \return 
	 *
	 * @param path
	 * @return
	 */
	@Override
	public InputStream getInputStream(String path) {
		try {
		
			if(!path.startsWith("/")) { path = "/"+path;}
			
			return this.davClient.getMethodData(path);
		}
		catch (Exception ex) {
			RaiseException(ex, SOSVfs_E_193.params("getInputStream()", path));
			return null;
		}
	}

	/**
	 * 
	 * \brief getOutputStream
	 * 
	 * \details
	 *
	 * \return 
	 *
	 * @param path
	 * @return
	 */
	@Override
	public OutputStream getOutputStream(String path) {
		WebdavResource res = null;
		try {
			res = this.getResource(path);
			return new SOSVfsWebDAVOutputStream(res);
		}
		catch (Exception ex) {
			RaiseException(ex, SOSVfs_E_193.params("getOutputStream()", path));
			return null;
		}
		finally {
			try {
				if (res != null) {
					res.close();
				}
			}
			catch (Exception e) {
			}
		}
	}

	/**
	 * 
	 * \brief changeWorkingDirectory
	 * 
	 * \details
	 *
	 * \return 
	 *
	 * @param path
	 * @return
	 */
	@Override
	public boolean changeWorkingDirectory(String path) {
		try {
			String origPath = this.davClient.getPath();

			path = this.normalizePath(path);

			this.davClient.setPath(path);
			if (this.davClient.exists()) {
				this.reply = "cwd OK";
				logDEBUG(SOSVfs_D_194.params(path, getReplyString()));
			}
			else {
				this.davClient.setPath(origPath);
				this.reply = "cwd failed";
				logDEBUG(SOSVfs_D_194.params(path, getReplyString()));
				return false;
			}

		}
		catch (Exception ex) {
			RaiseException(ex, SOSVfs_E_193.params("cwd", path));
		}
		return true;
	}

	/**
	 * 
	 * \brief getFileHandle
	 * 
	 * \details
	 *
	 * \return 
	 *
	 * @param fileName
	 * @return
	 */
	@Override
	public ISOSVirtualFile getFileHandle(String fileName) {
		fileName = adjustFileSeparator(fileName);
		ISOSVirtualFile file = new SOSVfsWebDAVFile(fileName);
		file.setHandler(this);

		logDEBUG(SOSVfs_D_196.params(fileName));

		return file;
	}

	/**
	 * 
	 * \brief getModificationTime
	 * 
	 * \details
	 *
	 * \return 
	 *
	 * @param path
	 * @return
	 */
	@Override
	public String getModificationTime(String path) {
		WebdavResource res = null;
		String dateTime = null;
		try {
			res = this.getResource(path);
			if (res.exists()) {
				long lm = res.getGetLastModified();
				DateFormat df = new SimpleDateFormat("yyyy-MM-dd HH:mm:ss");
				dateTime = df.format(new Date(lm));
			}
		}
		catch (Exception ex) {

		}
		finally {
			if (res != null) {
				try {
					res.close();
				}
				catch (Exception e) {
				}
			}
		}

		return dateTime;
	}

	/**
	 * 
	 * \brief fileExists
	 * 
	 * \details
	 *
	 * \return 
	 *
	 * @param filename
	 * @return
	 */
	@Override
	protected boolean fileExists(String filename) {

		WebdavResource res = null;
		try {
			res = this.getResource(filename);

			return res.exists();
		}
		catch (Exception e) {
			return false;
		}
		finally {
			try {
				if (res != null) {
					res.close();
				}
			}
			catch (Exception e) {
			}
		}
	}

	/**
	 * 
	 * \brief getCurrentPath
	 * 
	 * \details
	 *
	 * \return 
	 *
	 * @return
	 */
	protected String getCurrentPath() {
		String path = null;

		try {
			path = this.davClient.getPath();
			logDEBUG(HostID(SOSVfs_D_195.params(path)));
			LogReply();
		}
		catch (Exception e) {
			RaiseException(e, SOSVfs_E_134.params("getCurrentPath"));
		}

		return path;
	}

	/**
	 * 
	 * \brief getResource
	 * 
	 * \details
	 *
	 * \return WebdavResource
	 *
	 * @param path
	 * @return
	 * @throws Exception
	 */
	private WebdavResource getResource(String path) throws Exception {
		return new WebdavResource(this.rootUrl, this.normalizePath(path));
	}

	/**
	 * 
	 * \brief normalizePath
	 * 
	 * \details
	 *
	 * \return String
	 *
	 * @param path
	 * @return
	 */
	private String normalizePath(String path) {
		return path.startsWith("/") ? path : "/" + path;
	}

	/**
	 * 
	 * \brief setRootHttpURL
	 * 
	 * \details
	 *
	 * \return void
	 *
	 * @param puser
	 * @param ppassword
	 * @param phost
	 * @param pport
	 * @throws Exception
	 */
	private void setRootHttpURL(String puser, String ppassword, String phost, int pport) throws Exception {

		this.rootUrl = null;
		String path = "/";

		if (this.connection2OptionsAlternate.auth_method.isURL()) {
			if (phost.toLowerCase().startsWith("https://")) {
				this.rootUrl = new HttpsURL(phost);
			}
			else {
				this.rootUrl = new HttpURL(phost);
			}
		}
		else {
			this.rootUrl = new HttpURL(phost, pport, path);
		}

		this.rootUrl.setUserinfo(puser, ppassword);
	}

	/**
	 * 
	 * \brief doAuthenticate
	 * 
	 * \details
	 *
	 * \return ISOSConnection
	 *
	 * @param authenticationOptions
	 * @return
	 * @throws Exception
	 */
	private ISOSConnection doAuthenticate(ISOSAuthenticationOptions pAuthenticationOptions) throws Exception {

		this.authenticationOptions = pAuthenticationOptions;

		this.userName = this.authenticationOptions.getUser().Value();
		String password = this.authenticationOptions.getPassword().Value();
		logDEBUG(SOSVfs_D_132.params(this.userName));
		
		this.setRootHttpURL(this.userName, password, this.host, this.port);

		try {
			// eigentlich bereits bei new WebdavResource soll eine Exception ausgelst werden,
			// wenn die Credentials bzw host etc falsch sind
			// hier aber aus irgend. Grund kommt keine Exception hoch
			this.davClient = new WebdavResource(this.rootUrl);
			if (!this.davClient.exists()) {
				throw new JobSchedulerException("not connected " + this.davClient.getStatusMessage());
			}
		}
		catch (Exception ex) {
			throw new JobSchedulerException(SOSVfs_E_167.params(this.authenticationOptions.getAuth_method().Value(), this.authenticationOptions.getAuth_file()
					.Value()));
		}

		this.reply = "OK";
		logDEBUG(SOSVfs_D_133.params(this.userName));
		this.LogReply();

		return this;
	}

	/**
	 * 
	 * \brief connect
	 * 
	 * \details
	 *
	 * \return void
	 *
	 * @param phost
	 * @param pport
	 */
	private void connect(String phost, int pport) {

		this.host = phost;
		this.port = pport;

		// TODO Meldung auch ohne Port oder Port mit ?? ausgeben.
		String msgPort = (this.connection2OptionsAlternate.auth_method.isURL()) ? "??" : ""+this.port;
		
		logDEBUG(SOSVfs_D_0101.params(this.host, this.port));
		
		if (this.isConnected() == false) {

			this.LogReply();
		}
		else {
			logWARN(SOSVfs_D_0103.params(this.host, this.port));
		}
	}

	/**
	 * 
	 * @author Robert Ehrlich
	 *
	 */
	private class SOSVfsWebDAVOutputStream extends ByteArrayOutputStream {

		private WebdavResource	resource;

		/**
		 * 
		 * \brief SOSVfsWebDAVOutputStream
		 *
		 * \details
		 *
		 * @param res
		 */
		public SOSVfsWebDAVOutputStream(WebdavResource res) {
			super();
			this.resource = res;
		}

		/**
		 * 
		 * \brief close
		 * 
		 * \details
		 *
		 * \return 
		 *
		 * @throws IOException
		 */
		public void close() throws IOException {
			super.close();

			try {
				this.resource.putMethod(this.toByteArray());
			}
			catch (HttpException ex) {
				throw new IOException(ex);
			}
		}
	}

}
