/********************************************************* begin of preamble
**
** Copyright (C) 2003-2012 Software- und Organisations-Service GmbH. 
** All rights reserved.
**
** This file may be used under the terms of either the 
**
**   GNU General Public License version 2.0 (GPL)
**
**   as published by the Free Software Foundation
**   http://www.gnu.org/licenses/gpl-2.0.txt and appearing in the file
**   LICENSE.GPL included in the packaging of this file. 
**
** or the
**  
**   Agreement for Purchase and Licensing
**
**   as offered by Software- und Organisations-Service GmbH
**   in the respective terms of supply that ship with this file.
**
** THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS
** IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO,
** THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR
** PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS
** BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
** CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
** SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
** INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
** CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
** ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
** POSSIBILITY OF SUCH DAMAGE.
********************************************************** end of preamble*/
package com.sos.dailyschedule.dialog.classes;

import java.io.File;
import java.util.Date;


import org.eclipse.swt.widgets.Composite;

import org.eclipse.swt.custom.SashForm;
import org.eclipse.swt.SWT;
import org.eclipse.swt.layout.GridLayout;
import org.eclipse.swt.layout.GridData;
import org.eclipse.swt.widgets.Group;


import com.sos.dialog.interfaces.ITableView;
import com.sos.jobnet.dataprovider.JobNetDataProvider;
import com.sos.jobnet.dataprovider.JobNetNodeDataProvider;
import com.sos.jobnet.dataprovider.JobNetPlanDataProvider;

public class SOSDashboardViewJobNet  extends SOSDashboardMainView implements ITableView{

    private static final int    NUMBER_OF_COLUMNS_IN_GRID   = 9;

    private SOSDashboardTableViewJobNet tableViewJobNet;
    private SOSDashboardTableViewJobNet tableViewJobNetPlan;
    private SOSDashboardTableViewJobNetNodes tableViewJobNetNodes;
    private SOSDashboardTableViewJobNetBootstraps tableViewJobNetBootstraps;

    private File configurationFile;
    private JobNetPlanDataProvider jobNetPlanDataProvider = null;
    private JobNetNodeDataProvider jobNetNodeDataProvider = null;
    private JobNetDataProvider jobNetDataProvider = null;

    
    private Group grpJobnet;
    private Group grpNodes;
    private Group grpBootstraps;
    private Group grpPlans;


    public SOSDashboardViewJobNet (Composite parent) {
        super(parent);
    }

    
    
    /**
     * @wbp.parser.entryPoint
     */
    public void createTable  () {
        
        mainViewComposite = new Composite(leftTabFolder, SWT.NONE);
        GridLayout layout = new GridLayout(NUMBER_OF_COLUMNS_IN_GRID, false);
        mainViewComposite.setLayout(layout);
        sosDashboardHeader = new SosDashboardHeader(mainViewComposite, this);
         
        SashForm sashForm = new SashForm(mainViewComposite,  SWT.SMOOTH | SWT.HORIZONTAL);
        GridData gd_sashForm = new GridData(GridData.FILL_BOTH);
        gd_sashForm.horizontalSpan = 9;
        sashForm.setLayoutData(gd_sashForm);
        
        grpJobnet = new Group(sashForm, SWT.NONE);
        grpJobnet.setText("Jobnet");

        grpNodes = new Group(sashForm, SWT.NONE);
        grpNodes.setText("Nodes");
        
        grpBootstraps = new Group(sashForm, SWT.NONE);
        grpBootstraps.setText("Bootstraps");

        grpPlans = new Group(sashForm, SWT.NONE);
        grpPlans.setText("Plans");

        sashForm.setWeights(new int[] {1, 1, 1, 1});
       
       
        createTableJobnet();
        createTableJobNetNodes();
        createTableJobNetBootstrap(); 
        createTableJobNetPlan();
        
    }

    private void createTableJobnet(){
        jobNetDataProvider = new JobNetDataProvider(configurationFile);
        jobNetDataProvider.setFrom(new Date());
        jobNetDataProvider.setTo(new Date());
       // jobNetNodeDataProvider.getData();

        
        
        tableViewJobNet = new SOSDashboardTableViewJobNet(grpJobnet);
        tableViewJobNet.setObjOptions(objOptions);
        tableViewJobNet.mainViewComposite = grpJobnet;
        tableViewJobNet.setLeftTabFolder(leftTabFolder);
        tableViewJobNet.setPrefs(prefs); 
        tableViewJobNet.setRight(right);
        tableViewJobNet.setLeft(grpJobnet);
        tableViewJobNet.setTableDataProvider(jobNetDataProvider);
        tableViewJobNet.setDetailHistoryDataProvider(detailHistoryDataProvider);
        tableViewJobNet.sosDashboardHeader = sosDashboardHeader;
        tableViewJobNet.createTable();
        
     
    }
    
    private void createTableJobNetNodes(){
        jobNetNodeDataProvider = new JobNetNodeDataProvider(configurationFile);
        jobNetNodeDataProvider.setFrom(new Date());
        jobNetNodeDataProvider.setTo(new Date());
        //jobNetNodeDataProvider.getData();

        
        
        tableViewJobNetNodes = new SOSDashboardTableViewJobNetNodes(grpNodes);
        tableViewJobNetNodes.setObjOptions(objOptions);
        tableViewJobNetNodes.mainViewComposite = grpNodes;
        tableViewJobNetNodes.setLeftTabFolder(leftTabFolder);
        tableViewJobNetNodes.setPrefs(prefs); 
        tableViewJobNetNodes.setRight(right);
        tableViewJobNetNodes.setLeft(grpNodes);
        tableViewJobNetNodes.setTableDataProvider(jobNetNodeDataProvider);
        tableViewJobNetNodes.setDetailHistoryDataProvider(detailHistoryDataProvider);
        tableViewJobNetNodes.sosDashboardHeader = sosDashboardHeader;
        tableViewJobNetNodes.createTable();
    }
    
    private void createTableJobNetBootstrap(){
        jobNetPlanDataProvider = new JobNetPlanDataProvider(configurationFile,"");
        jobNetPlanDataProvider.setFrom(new Date());
        jobNetPlanDataProvider.setTo(new Date());
        jobNetPlanDataProvider.getData();

        
        
        tableViewJobNetBootstraps = new SOSDashboardTableViewJobNetBootstraps(grpBootstraps);
        tableViewJobNetBootstraps.setObjOptions(objOptions);
        tableViewJobNetBootstraps.mainViewComposite = grpBootstraps;
        tableViewJobNetBootstraps.setLeftTabFolder(leftTabFolder);
        tableViewJobNetBootstraps.setPrefs(prefs); 
        tableViewJobNetBootstraps.setRight(right);
        tableViewJobNetBootstraps.setLeft(grpBootstraps);
        tableViewJobNetBootstraps.setTableDataProvider(jobNetPlanDataProvider);
        tableViewJobNetBootstraps.setDetailHistoryDataProvider(detailHistoryDataProvider);
        tableViewJobNetBootstraps.sosDashboardHeader = sosDashboardHeader;
        tableViewJobNetBootstraps.createTable();
    }
    
    private void createTableJobNetPlan(){
        jobNetPlanDataProvider = new JobNetPlanDataProvider(configurationFile,"");
        jobNetPlanDataProvider.setFrom(new Date());
        jobNetPlanDataProvider.setTo(new Date());
        jobNetPlanDataProvider.getData();

        
        
        tableViewJobNet = new SOSDashboardTableViewJobNet(grpPlans);
        tableViewJobNet.setObjOptions(objOptions);
        tableViewJobNet.mainViewComposite = grpPlans;
        tableViewJobNet.setLeftTabFolder(leftTabFolder);
        tableViewJobNet.setPrefs(prefs); 
        tableViewJobNet.setRight(right);
        tableViewJobNet.setLeft(grpPlans);
        tableViewJobNet.setTableDataProvider(jobNetPlanDataProvider);
        tableViewJobNet.setDetailHistoryDataProvider(detailHistoryDataProvider);
        tableViewJobNet.sosDashboardHeader = sosDashboardHeader;
        tableViewJobNet.createTable();
    }
    
    
    

    @Override public void getTableData() {
        tableViewJobNet.getTableData();
        
    }



    @Override public void buildTable() {
        tableViewJobNet.buildTable();
        
    }



    @Override public void createMenue() {
        tableViewJobNet.createMenue();
    }



    @Override public void getList() {
        tableViewJobNet.getList();
    }



    @Override public void actualizeList() {
       tableViewJobNet.actualizeList();
    }



    public void setConfigurationFile(File configurationFile) {
        this.configurationFile = configurationFile;
    }




        
}
