/********************************************************* begin of preamble
**
** Copyright (C) 2003-2012 Software- und Organisations-Service GmbH. 
** All rights reserved.
**
** This file may be used under the terms of either the 
**
**   GNU General Public License version 2.0 (GPL)
**
**   as published by the Free Software Foundation
**   http://www.gnu.org/licenses/gpl-2.0.txt and appearing in the file
**   LICENSE.GPL included in the packaging of this file. 
**
** or the
**  
**   Agreement for Purchase and Licensing
**
**   as offered by Software- und Organisations-Service GmbH
**   in the respective terms of supply that ship with this file.
**
** THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS
** IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO,
** THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR
** PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS
** BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
** CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
** SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
** INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
** CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
** ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
** POSSIBILITY OF SUCH DAMAGE.
********************************************************** end of preamble*/
package com.sos.jobnet.waiter;
import java.io.File;
import java.util.Date;
import java.util.List;

import org.apache.log4j.Logger;

import com.sos.JSHelper.io.Files.JSFile;
import com.sos.i18n.I18NBase;
import com.sos.i18n.annotation.I18NResourceBundle;
import com.sos.jobnet.classes.JobNetCollection;
import com.sos.jobnet.classes.JobNetConstants;
import com.sos.jobnet.classes.JobNetException;
import com.sos.jobnet.classes.Status;
import com.sos.jobnet.classes.Status.enuStatus;
import com.sos.jobnet.db.JobNetEdgesDBItem;
import com.sos.jobnet.db.JobNetEdgesDBLayer;
import com.sos.jobnet.db.JobNetPlanDBItem;
import com.sos.jobnet.db.JobNetPlanDBLayer;
import com.sos.jobnet.interfaces.IJobNetCollection;
import com.sos.jobnet.interfaces.IWaitHandler;
import com.sos.jobnet.options.JobNetOptions;

/**
* \class JobNetDBWaiter 
* 
* \brief JobNetDBWaiter - 
* 
* \details
*
* \section JobNetDBWaiter.java_intro_sec Introduction
*
* \section JobNetDBWaiter.java_samples Some Samples
*
* \code
*   .... code goes here ...
* \endcode
*
* <p style="text-align:center">
* <br />---------------------------------------------------------------------------
* <br /> APL/Software GmbH - Berlin
* <br />##### generated by ClaviusXPress (http://www.sos-berlin.com) #########
* <br />---------------------------------------------------------------------------
* </p>
* \author oh
* @version $Id: JobNetDBWaiter.java 16850 2012-03-21 12:14:23Z oh $
* \see reference
*
* Created on 22.02.2012 13:56:28
 */
/**
 * @author oh
 *
 */
@I18NResourceBundle(baseName = "com_sos_jobnet_messages", defaultLocale = "en")
public class JobNetDBWaiter extends I18NBase implements IWaitHandler {
	private final String		conClassName			= "JobNetDBWaiter";
	private	final String		conSVNVersion			= "$Id: JobNetDBWaiter.java 16850 2012-03-21 12:14:23Z oh $";
	private static final Logger	logger					= Logger.getLogger(JobNetDBWaiter.class);
	private JobNetOptions		objOptions				= null;
	private JobNetPlanDBItem	curNode					= null;
	private JobNetPlanDBLayer	jobNetPlanDBLayer		= null;
	private List<JobNetEdgesDBItem> jobNetEdgesList		= null;
	private IJobNetCollection 	waitforNodes 			= new JobNetCollection();
	private boolean			 	curNodeFound	 		= true;
	
	private static final String	JOBNETDBW_E_0010		= "JOBNETDBW_E_0010";		// %s: No item found for the current node (%s) in database table SCHEDULER_JOB_NET_PLAN.
	private static final String	JOBNETDBW_E_0011		= "JOBNETDBW_E_0011";		// %s: No predecessor with node_id=%d found in database table SCHEDULER_JOB_NET_PLAN.
	

	public JobNetDBWaiter() {
		super(JobNetConstants.strBundleBaseName);
	}
	
	public JobNetDBWaiter(JobNetOptions jobNetOptions) {
		super(JobNetConstants.strBundleBaseName);
		setOptions(jobNetOptions);
	}
	
	public JobNetOptions Options() {
		@SuppressWarnings("unused")
		final String conMethodName = conClassName + "::Options";
		if (objOptions == null) {
			objOptions = new JobNetOptions();
		}
		return objOptions;
	} // public JobNetOptions Options

	
	/**
	 * \brief setOptions - 
	 *
	 * \details
	 * setter 
	 *
	 * @param objOptions the value for objOptions to set
	 */
	@SuppressWarnings("hiding")
	public void setOptions(JobNetOptions objOptions) {
		this.objOptions = objOptions;
	}
	
	
	/**
	 * \brief getCurNode
	 * 
	 * \details
	 */
	private JobNetPlanDBItem getCurNode() {
		final String conMethodName = conClassName + "::getCurNode";
		if (curNode == null && curNodeFound) {
			String uuidJobnetIdentifier 			= Options().uuid_jobnet_identifier.Value();
			String jobNet 							= Options().jobnet.Value();
			String node 							= Options().node.Value();
			String schedulerId 						= Options().scheduler_id.Value();
			JSFile hibernateConnectionConfigFile 	= Options().hibernate_connection_config_file.JSFile();
			String curNodeTitle 					= uuidJobnetIdentifier+"/"+schedulerId+":"+jobNet+":"+node;
			
			logger.debug(Messages.getMsg(JobNetConstants.JOBNET_D_0001, conMethodName, "uuid_jobnet_identifier", uuidJobnetIdentifier));
			logger.debug(Messages.getMsg(JobNetConstants.JOBNET_D_0001, conMethodName, "jobnet", jobNet));
			logger.debug(Messages.getMsg(JobNetConstants.JOBNET_D_0001, conMethodName, "node", node));
			logger.debug(Messages.getMsg(JobNetConstants.JOBNET_D_0001, conMethodName, "scheduler_id", schedulerId));
			logger.debug(Messages.getMsg(JobNetConstants.JOBNET_D_0001, conMethodName, "hibernate_connection_config_file", hibernateConnectionConfigFile.getAbsolutePath()));
			
			this.jobNetPlanDBLayer 	= new JobNetPlanDBLayer(hibernateConnectionConfigFile, schedulerId);
			
			jobNetPlanDBLayer.getFilter().setOrderCriteria(null);
			jobNetPlanDBLayer.getFilter().setUuid(uuidJobnetIdentifier);
			jobNetPlanDBLayer.getFilter().setJobnet(jobNet);
			jobNetPlanDBLayer.getFilter().setNode(node);
			
			List<JobNetPlanDBItem> jobNetPlanDBItem = jobNetPlanDBLayer.getJobnetPlanListWithLock(0);
			
			if(jobNetPlanDBItem == null || jobNetPlanDBItem.isEmpty()) {
				curNodeFound = false;
				String msg = Messages.getMsg(JOBNETDBW_E_0010, conMethodName, curNodeTitle);
				logger.error(msg);
				throw new JobNetException(msg);
			}
			else {
				curNode = jobNetPlanDBItem.get(0);
				logger.debug(Messages.getMsg(JobNetConstants.JOBNET_D_0010, conMethodName, curNode.getTitle()));
			}
		}
		return curNode;
	} // public JobNetPlanDBItem getCurNode
	
	
	
	/**
	 * \brief refreshCurNode
	 * 
	 * \details
	 */
	private JobNetPlanDBItem refreshCurNode() {
		final String conMethodName = conClassName + "::refreshCurNode";
		if (curNode != null && jobNetPlanDBLayer != null) {
			jobNetPlanDBLayer.getFilter().setNodeId(curNode.getNodeId());
			
			List<JobNetPlanDBItem> jobNetPlanDBItem = jobNetPlanDBLayer.getJobnetPlanListWithLock(0);
			
			if(jobNetPlanDBItem == null || jobNetPlanDBItem.isEmpty()) {
				curNodeFound = false;
				String msg = Messages.getMsg(JOBNETDBW_E_0010, conMethodName, curNode.getTitle());
				logger.error(msg);
				throw new JobNetException(msg);
			}
			else {
				curNode = jobNetPlanDBItem.get(0);
			}
		}
		return curNode;
	} // public JobNetPlanDBItem refreshCurNode
	
	
	/**
	 * \brief getJobnetEdgesList
	 * 
	 * \details
	 */
	private List<JobNetEdgesDBItem> getJobnetEdgesList() {
		@SuppressWarnings("unused")
		final String conMethodName = conClassName + "::getJobnetEdgesList";
		getCurNode();
		if (jobNetEdgesList == null) {
			File configurationFile 	= new File(Options().hibernate_connection_config_file.Value());
			JobNetEdgesDBLayer jobNetEdgesDBLayer = new JobNetEdgesDBLayer(configurationFile);
			jobNetEdgesList = jobNetEdgesDBLayer.getPredecessors(curNode.getJobnetNodeDBItem());
		}
		return jobNetEdgesList;
	} // public List<JobNetEdgesDBItem> getJobnetEdgesList
	
	
	/**
	 * \brief updateWaiterStatus
	 * 
	 * \details
	 *
	 * @param status
	 */
	private void updateWaiterStatus(enuStatus status) {
		final String conMethodName = conClassName + "::updateWaiterStatus";
		getCurNode();
		if (curNode!=null) {
			curNode.setStatusWaiter(status.value());
			logger.debug(Messages.getMsg(JobNetConstants.JOBNET_D_0011, conMethodName, Messages.getMsg(JobNetConstants.JOBNET_T_0010), Messages.getMsg(status.msgKey())));
//			after each updateWaiterStatus follows updateNodeStatus which calls the commit, 
//			so that only one sql transaction is used (see JobNetWaiter::run)
//			update(new Date());
		}
	} // private void updateWaiterStatus
	
	
	/**
	 * \brief updateRunnerStatus
	 * 
	 * \details
	 *
	 * @param status
	 */
	private void updateRunnerStatus(enuStatus status) {
		final String conMethodName = conClassName + "::updateRunnerStatus";
		getCurNode();
		if (curNode!=null) {
			curNode.setStatusRunner(status.value());
			logger.debug(Messages.getMsg(JobNetConstants.JOBNET_D_0011, conMethodName, Messages.getMsg(JobNetConstants.JOBNET_T_0011), Messages.getMsg(status.msgKey())));
//			after each updateRunnerStatus follows updateNodeStatus which calls the commit, 
//			so that only one sql transaction is used (see JobNetWaiter::run)
//			update(new Date()); 
		}
	} // private void updateRunnerStatus
	
	
	/**
	 * \brief updateNodeStatus
	 * 
	 * \details
	 *
	 * @param status
	 */
	private void updateNodeStatus(enuStatus status) {
		final String conMethodName = conClassName + "::updateNodeStatus";
		getCurNode();
		if (curNode!=null) {
			curNode.setStatusNode(status.value());
			logger.debug(Messages.getMsg(JobNetConstants.JOBNET_D_0011, conMethodName, Messages.getMsg(JobNetConstants.JOBNET_T_0013), Messages.getMsg(status.msgKey())));
			update(new Date());
		}
	} // private void updateNodeStatus
	
	
	/**
	 * \brief update
	 * 
	 * \details
	 */
	private void update(Date tstamp) {
		@SuppressWarnings("unused")
		final String conMethodName = conClassName + "::update";
		curNode.setModified(tstamp);
		curNode.setModifiedBy(JobNetDBWaiter.class.getName());
		jobNetPlanDBLayer.beginTransaction();
		jobNetPlanDBLayer.saveOrUpdate(curNode);
		jobNetPlanDBLayer.commit();
	} // private void update
	
	
	private String getI18NYesNo(boolean flgIsYes) {
		@SuppressWarnings("unused")
		final String conMethodName = conClassName + "::getI18NYesNo";
		String msg = Messages.getMsg(JobNetConstants.JOBNET_T_0002); //No
		if (flgIsYes) {
			msg = Messages.getMsg(JobNetConstants.JOBNET_T_0001); //Yes
		}
		return msg;
	}

	
	/**
	 * \brief isBootstrap
	 * 
	 * \details
	 *
	 * \return 
	 *
	 * @return boolean
	 */
	@Override
	public boolean isBootstrap() {
		final String conMethodName = conClassName + "::isBootstrap";
		boolean isBootstrap = getCurNode().getBootstrap();
		logger.debug(Messages.getMsg(JobNetConstants.JOBNET_D_0021, conMethodName, getI18NYesNo(isBootstrap)));
		return isBootstrap;
	} // public boolean isBootstrap

	
	/**
	 * \brief setWaiterStatusToRunning
	 * 
	 * \details
	 */
	@Override
	public void setWaiterStatusToRunning() {
		updateWaiterStatus(enuStatus.RUNNING);
	}

	/**
	 * \brief setNodeStatusToRunning
	 * 
	 * \details
	 */
	@Override
	public void setNodeStatusToRunning() {
		updateNodeStatus(enuStatus.RUNNING);
	}
	


	/**
	 * \brief isWaiterSkipped
	 * 
	 * \details
	 *
	 * \return 
	 *
	 * @return boolean
	 */
	@Override
	public boolean isWaiterSkipped() {
		final String conMethodName = conClassName + "::isWaiterSkipped";
		boolean isSkipped = getCurNode().getIsWaiterSkipped();
		logger.debug(Messages.getMsg(JobNetConstants.JOBNET_D_0020, conMethodName, Messages.getMsg(JobNetConstants.JOBNET_T_0010), getI18NYesNo(isSkipped)));
		return isSkipped;
	}
	

	/**
	 * \brief isRunnerSkipped
	 * 
	 * \details
	 *
	 * \return 
	 *
	 * @return boolean
	 */
	@Override
	public boolean isRunnerSkipped() {
		final String conMethodName = conClassName + "::isRunnerSkipped";
		boolean isSkipped = getCurNode().getIsRunnerSkipped();
		logger.debug(Messages.getMsg(JobNetConstants.JOBNET_D_0020, conMethodName, Messages.getMsg(JobNetConstants.JOBNET_T_0011), getI18NYesNo(isSkipped)));
		return isSkipped;
	}
	
	/**
	 * \brief isRunnerRunning
	 * 
	 * \details
	 *
	 * \return 
	 *
	 * @return boolean
	 */
	@Override
	public boolean isRunnerRunning() {
		@SuppressWarnings("unused")
		final String conMethodName = conClassName + "::isRunnerRunning";
		boolean isRunning = new Status().isRunning(getCurNode().getStatusRunner());
		return isRunning;
	}
	
	/**
	 * \brief isRunnerOnDemand
	 * 
	 * \details
	 *
	 * \return 
	 *
	 * @return boolean
	 */
	@Override
	public boolean isRunnerOnDemand() {
		final String conMethodName = conClassName + "::isRunnerOnDemand";
		boolean isOnDemand = getCurNode().getIsRunnerOnDemand();
		logger.debug(Messages.getMsg(JobNetConstants.JOBNET_D_0022, conMethodName, Messages.getMsg(JobNetConstants.JOBNET_T_0011), getI18NYesNo(isOnDemand)));
		return isOnDemand;
	}

	/**
	 * \brief setNodeStatusToWaiting
	 * 
	 * \details
	 */
	@Override
	public void setNodeStatusToWaiting() {
		updateNodeStatus(enuStatus.WAITING);
	}

	/**
	 * \brief setRunnerStatusToRunning
	 * 
	 * \details
	 */
	@Override
	public void setRunnerStatusToRunning() {
		updateRunnerStatus(enuStatus.RUNNING);
	}

	/**
	 * \brief setWaiterStatusToFinish
	 * 
	 * \details
	 */
	@Override
	public void setWaiterStatusToFinish() {
		updateWaiterStatus(enuStatus.FINISHED);
	}

	/**
	 * \brief setWaiterStatusToError
	 * 
	 * \details
	 */
	@Override
	public void setWaiterStatusToError() {
		updateWaiterStatus(enuStatus.ERROR);
	}

	/**
	 * \brief setNodeStatusToError
	 * 
	 * \details
	 */
	@Override
	public void setNodeStatusToError() {
		updateNodeStatus(enuStatus.ERROR);
	}
	
	/**
	 * \brief setStartTime
	 * 
	 * \details
	 */
	@Override
	public void setStartTime() {
		@SuppressWarnings("unused")
		final String conMethodName = conClassName + "::setStartTime";
		getCurNode();
		if (curNode!=null) {
			Date now = new Date();
			curNode.setStartTime(now);
//			after each setStartTime follows updateNodeStatus which calls the commit, 
//			so that only one sql transaction is used (see JobNetWaiter::run)
//			update(now); 
		}
	}

	@Override
	public int getNumberOfWaitingNodes() {
		if(waitforNodes.size() < 0) {
			getWaitingNodes();
		}
		return waitforNodes.size();
	}
	
	@Override
	public IJobNetCollection getWaitingNodes() {
		final String conMethodName = conClassName + "::getWaitingNodes";
		getJobnetEdgesList();
		
		int status = -1;
		Status objStatus = new Status();
		List<JobNetPlanDBItem> jobNetPlanDBItem = null;
		JobNetPlanDBItem jobnetPlanDBItem = null;
		waitforNodes = new JobNetCollection();
		
		for (JobNetEdgesDBItem jobnetEdgesItem : jobNetEdgesList) {
			jobNetPlanDBLayer.getFilter().setNode(null);
			jobNetPlanDBLayer.getFilter().setNodeId(jobnetEdgesItem.getChildNodeId());
			jobNetPlanDBItem = jobNetPlanDBLayer.getJobnetPlanList(0);
			
			if(jobNetPlanDBItem == null || jobNetPlanDBItem.isEmpty()) {
				String msg = Messages.getMsg(JOBNETDBW_E_0011, conMethodName, jobnetEdgesItem.getChildNodeId()); 
				logger.error(msg);
				throw new JobNetException(msg);
			}
			
			jobnetPlanDBItem = jobNetPlanDBItem.get(0);
			String predecessor = jobnetPlanDBItem.getJobnetNodeDBItem().getNode();
			status = jobnetPlanDBItem.getStatusNode();
			logger.info(Messages.getMsg(JobNetConstants.JOBNETW_I_0010, conMethodName, predecessor, objStatus.getMsg(status)));
			if (objStatus.isFinished(status)) {
				continue;
			}
			waitforNodes.add(jobnetPlanDBItem);
		}
		return waitforNodes;
	}

	@Override
	public void open() {
		logger.info(String.format("%1s %2s", conClassName, conSVNVersion));
		getCurNode();
		//TODO skips in Options lesen und in DB setzen
		//Ist unntig, wenn das dashboard die Skip-Funktionalitt zur Verfgung stellt
	}

	@Override
	public void close() {
	}

	@Override
	public void refresh() {
		final String conMethodName = conClassName + "::refresh";
		refreshCurNode();
	}

}
