/********************************************************* begin of preamble
**
** Copyright (C) 2003-2012 Software- und Organisations-Service GmbH. 
** All rights reserved.
**
** This file may be used under the terms of either the 
**
**   GNU General Public License version 2.0 (GPL)
**
**   as published by the Free Software Foundation
**   http://www.gnu.org/licenses/gpl-2.0.txt and appearing in the file
**   LICENSE.GPL included in the packaging of this file. 
**
** or the
**  
**   Agreement for Purchase and Licensing
**
**   as offered by Software- und Organisations-Service GmbH
**   in the respective terms of supply that ship with this file.
**
** THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS
** IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO,
** THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR
** PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS
** BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
** CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
** SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
** INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
** CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
** ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
** POSSIBILITY OF SUCH DAMAGE.
********************************************************** end of preamble*/
package com.sos.scheduler.engine.kernel.scripting;

import java.util.regex.Matcher;
import java.util.regex.Pattern;

/**
 * \file ScriptFunction.java
 * \brief class for handling function names
 *  
 * \class ScriptFunction
 * \brief class for handling function names
 * 
 * \details
 * This class provide different version of function names:
 * getFunctionName(): the name of the function followed by braces, e.g. myFunction()
 * getNativeFunctionName(): the pure name of the function, e.g. myFunction()
 * 
 * Additionally the method isFunction() is provided for testing the existence of the
 * function in a given script.
 * 
 * With getTypeId() you can get the COM-type of the function.
 * \code
  \endcode
 *
 * \author ss
 * \version 1.0 - 18.01.2011 11:20:08
 * <div class="sos_branding">
 *   <p>(c) 2011 SOS GmbH - Berlin (<a style='color:silver' href='http://www.sos-berlin.com'>http://www.sos-berlin.com</a>)</p>
 * </div>
 */
public class ScriptFunction {

	private String	functionName	= "";
	private String	typeId			= "";
	
	public ScriptFunction(String fullName) {
		boolean propsToSet = true;
		if (fullName.contains(")") && !fullName.endsWith(")")) {
			Pattern p = Pattern.compile("^(.*)([^\\)])$");
			Matcher m = p.matcher(fullName);
			if (m.matches()) {
				functionName = m.group(1);
				typeId = m.group(2);
				propsToSet = false;
			}
		}
		if (propsToSet) {
			functionName = fullName;
			functionName = getNativeFunctionName() + "()";
			typeId = "V"; // void
		}
	}

	public String getFunctionName() {
		return functionName;
	}

	public String getNativeFunctionName() {
		return functionName.replaceAll("\\(.*\\)", "");
	}

	public String getTypeId() {
		return typeId;
	}
	
	/**
	 * \brief check i a function exists in the given code
	 * \detail
	 * the presence of a function is determined with the functionname followed by optional whitespaces 
	 * and an opened brace sign, that means ( or { or [.
	 * 
	 * @param scriptCode
	 * @return
	 */
	public boolean isFunction(String scriptCode) {
		final Pattern p = Pattern.compile( getNativeFunctionName() + "\\s*(\\(|\\{|\\[)");
		Matcher m = p.matcher(scriptCode);
		return m.find();
	}

}
