/********************************************************* begin of preamble
**
** Copyright (C) 2003-2012 Software- und Organisations-Service GmbH. 
** All rights reserved.
**
** This file may be used under the terms of either the 
**
**   GNU General Public License version 2.0 (GPL)
**
**   as published by the Free Software Foundation
**   http://www.gnu.org/licenses/gpl-2.0.txt and appearing in the file
**   LICENSE.GPL included in the packaging of this file. 
**
** or the
**  
**   Agreement for Purchase and Licensing
**
**   as offered by Software- und Organisations-Service GmbH
**   in the respective terms of supply that ship with this file.
**
** THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS
** IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO,
** THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR
** PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS
** BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
** CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
** SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
** INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
** CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
** ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
** POSSIBILITY OF SUCH DAMAGE.
********************************************************** end of preamble*/
package com.sos.scheduler.engine.kernel.util.sync;

import java.util.concurrent.ArrayBlockingQueue;
import java.util.concurrent.BlockingQueue;
import java.util.concurrent.TimeUnit;

import javax.annotation.Nullable;

import com.sos.scheduler.engine.kernel.util.Time;

public class Gate<T> {
    private final BlockingQueue<T> queue = new ArrayBlockingQueue<T>(1);

    public void put(T o) throws InterruptedException {
        queue.put(o);
    }

    public boolean offer(T o)  {
        return queue.offer(o);
    }

    public void expect(T o, Time t) throws InterruptedException {
        T result = tryPoll(t);
        if (result == null)  throw new RuntimeException("Expected message '"+o+"' has not arrived within "+t);
        if (!result.equals(o))  throw new RuntimeException("Message '"+o+"' has been expected, but '"+result+"' arrived");
    }

    public T poll(Time t) throws InterruptedException {
        T result = tryPoll(t);
        if (result == null)  throw new RuntimeException("Expected message has not arrived within "+t);
        return result;
    }

    @Nullable private T tryPoll(Time t) throws InterruptedException {
        return queue.poll(t.getMillis(), TimeUnit.MILLISECONDS);
    }
}
