/********************************************************* begin of preamble
**
** Copyright (C) 2003-2012 Software- und Organisations-Service GmbH. 
** All rights reserved.
**
** This file may be used under the terms of either the 
**
**   GNU General Public License version 2.0 (GPL)
**
**   as published by the Free Software Foundation
**   http://www.gnu.org/licenses/gpl-2.0.txt and appearing in the file
**   LICENSE.GPL included in the packaging of this file. 
**
** or the
**  
**   Agreement for Purchase and Licensing
**
**   as offered by Software- und Organisations-Service GmbH
**   in the respective terms of supply that ship with this file.
**
** THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS
** IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO,
** THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR
** PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS
** BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
** CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
** SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
** INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
** CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
** ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
** POSSIBILITY OF SUCH DAMAGE.
********************************************************** end of preamble*/
// $Id: Memory_allocator.h 14145 2010-12-02 12:11:47Z jz $

#ifndef __ZSCHIMMER_MEMORY_ALLOCATOR_H
#define __ZSCHIMMER_MEMORY_ALLOCATOR_H

#include "mutex.h"



namespace zschimmer {


struct Memory_allocator
{
    static bool                 debug_memory            ();
    static void*                z_malloc                ( size_t size, const char* name, int lineno );
    static void                 z_free                  ( void* p );

    struct Memory_type {
        Memory_type(const char* name, int line_no, size_t size) : _name(name),_line_no(line_no), _size(size) {}
        string to_string() const { return S() << _name << ":" << _line_no; }
        const char* _name;
        int _line_no;
        size_t _size;
    };

    struct Allocation {
        Allocation() : _count(0) {}
        int _count;
    };

    typedef std::map<Memory_type,Allocation> Allocation_map;
    static Allocation_map allocation_map();
    static bool static_debug_memory;
    static bool static_initialized;

private:
    static Mutex _mutex;

    typedef stdext::hash_map<void*,Memory_type>  Map;
    static Map _map;
};


inline size_t hash_value(const Memory_allocator::Memory_type& o) { 
    return (size_t)o._name ^ o._line_no ^ o._size; 
}

inline bool operator<(const Memory_allocator::Memory_type& a, const Memory_allocator::Memory_type& b) { 
    int c = strcmp(a._name, b._name); 
    return c < 0 || (c == 0 && a._size < b._size); 
} 

}

#endif
